/***************************************
    GameManager Object -- manages and
        runs the game
***************************************/

function GameManager(p1, p2) {

    // Initialize the player objects
    this.player1 = p1;
    this.player2 = p2;
    
    // get the canvas element from the HTML page
    this.canvasElem = document.getElementById("gameCanvas");
    this.context = this.canvasElem.getContext("2d");
    
    // store the game board
    this.board = new Board(this.canvasElem.width/8);
    
    // more game information
    this.turn = null;
    this.selectedPiece = null;
    this.moves = null;
    this.jumps = null;
    
    // for double jumps
    this.ignoreInput = false;
}

// Function for starting the game
GameManager.prototype.startGame = function() {
    
    // draw the board to the screen, write the scores
    this.board.draw(this.context);
    
    // get a copy of the game manager state
    var manager = this;
    
    // add an event listener for mouse click
    this.canvasElem.addEventListener('mousedown', function(e) {
        for (var i = 0; i < 8; i++) {
            for (var j = 0; j < 8; j++) {
                // check each piece to see if it has been selected
                if (manager.board.board[i][j].isSelected(e.offsetY-5, e.offsetX-5) == true) {
                    // set the selected piece
                    manager.selectPiece(i, j, manager.turn);
                    if (manager.selectedPiece == null) {
                        console.log("User has de-selected their piece.");
                    }
                    else {
                        console.log("User has selected piece at: "+manager.selectedPiece.x+", "+manager.selectedPiece.y);
                    }
                }
            }
        }
    }, true);
    
    // write the score
    this.writeScore();
    
    // reset players
    this.player1.numPieces = 12;
    this.player2.numPieces = 12;
    
    // set the turn
    this.turn = 1;    
}


// Select the specified piece
GameManager.prototype.selectPiece = function(posX, posY, turn) {
    // get the square we have selected
    var square = this.board.board[posX][posY];
    
    // check if the user has selected a legitimate piece
    if (square.hasPiece == true && this.ignoreInput == false) {
        if (square.piece.colour === "W" && turn === 1) {
            // set selected piece, and get all possible moves/jumps
            this.selectedPiece = square;
            this.moves = this.getMoves(this.selectedPiece, turn);
            this.jumps = this.getJumps(this.selectedPiece, turn);
        }
        else if (square.piece.colour === "B" && turn === 2) {
            // set selected piece, and get all possible moves/jumps
            this.selectedPiece = square;
            this.moves = this.getMoves(this.selectedPiece, turn);
            this.jumps = this.getJumps(this.selectedPiece, turn);  
        }
        else {   
            this.resetCanvas();
            this.selectedPiece = null;
        }
        if (this.selectedPiece != null) {       
            this.resetCanvas();     
            // display all possible moves
            this.board.drawMoves(this.moves, this.context);
            
            // display all possible jumps
            this.board.drawJumps(this.jumps, this.context);
        }
    }
    else {
        // check if the user is moving a piece
        if (this.selectedPiece != null) {
            if (this.moves.length > 0 && this.ignoreInput == false) {
                var validMove = false;
                for (var i = 0; i < this.moves.length; i++) {
                    // find a valid move
                    if (posX == this.moves[i].x && posY == this.moves[i].y) {
                        validMove = true;
                        var moveToX = this.moves[i].x;
                        var moveToY = this.moves[i].y;
                    }
                }
                
                // if there was a valid move selected, move the piece there,
                // increment the turn
                if (validMove == true) {
                    var turnDisplay = document.getElementById("turn");
                    // switch the turn
                    if (this.turn == 1) {
                        turnDisplay.innerHTML = "<h3>It is "+this.player2.name+"'s turn.</h3>";
                        this.turn = 2;
                    }
                    else {
                        turnDisplay.innerHTML = "<h3>It is "+this.player1.name+"'s turn.</h3>";
                        this.turn = 1;
                    }
                    
                    // move the piece
                    this.selectedPiece.piece.x = moveToY*this.selectedPiece.width+(this.selectedPiece.width/2);
                    this.selectedPiece.piece.y = moveToX*this.selectedPiece.width+(this.selectedPiece.width/2);
                    
                    console.log(moveToX);
                    // check if the piece is a king
                    if (turn == 1) {
                        if (moveToX == 7) {
                            this.selectedPiece.piece.isKing = true;
                        }
                    }
                    else if (turn == 2) {
                        if (moveToX == 0) {
                            this.selectedPiece.piece.isKing = true;
                        }
                    }
                    
                    // move the piece on the board
                    this.board.board[moveToX][moveToY].piece = this.board.board[this.selectedPiece.x][this.selectedPiece.y].piece;
                    this.board.board[this.selectedPiece.x][this.selectedPiece.y].piece = null;
                    this.board.board[this.selectedPiece.x][this.selectedPiece.y].hasPiece = false;
                    this.board.board[moveToX][moveToY].hasPiece = true;
                        
                    // redraw the canvas
                    this.resetCanvas(this.context);
                        
                    // un-select the piece
                    this.selectedPiece = null;
                    
                    /* THIS IS NOT WORKING
                    // check for a stalemate
                    var stale = false;
                    if (turn == 1) {
                        stale = this.checkStale(this.player2);
                        if (stale == true) {
                            alert(this.player1.name+" has won!");
                            this.player1.numWins++;
                            saveGame(this.player1, this.player2);
                            
                            // restart the game
                            init();
                        }
                    }
                    else if (turn == 2) {
                        stale = this.checkStale(this.player1);
                        if (stale == true) {
                            alert(this.player2.name+" has won!");
                            this.player2.numWins++;
                            saveGame(this.player1, this.player2);
                            
                            // restart the game
                            init();
                        }
                    }   */
                }
            }
            if (this.jumps.length > 0) {
                var validJump = false;
                for (var i = 0; i < this.jumps.length; i++) {
                    // find a valid jump
                    if (posX == this.jumps[i].x && posY == this.jumps[i].y) {
                        validJump = true;
                        var moveToX = this.jumps[i].x;
                        var moveToY = this.jumps[i].y;
                        var oppX = this.jumps[i].oppX;
                        var oppY = this.jumps[i].oppY;
                    }
                }
                
                // if there was a valid jump selected, move the piece there,
                // eliminate the opponent's piece, increment the turn
                if (validJump == true) {
                                    
                    // decrement the number of pieces the player controls
                    if (this.selectedPiece.piece.colour == "W") {
                        this.player2.numPieces--;
                    }
                    else if (this.selectedPiece.piece.colour == "B") {
                        this.player1.numPieces--;
                    }
                    
                    // move the piece
                    this.selectedPiece.piece.x = moveToY*this.selectedPiece.width+(this.selectedPiece.width/2);
                    this.selectedPiece.piece.y = moveToX*this.selectedPiece.width+(this.selectedPiece.width/2);
                    
                    console.log(moveToX);
                    // check if the piece is a king
                    if (turn == 1) {
                        if (moveToX == 7) {
                            this.selectedPiece.piece.isKing = true;
                        }
                    }
                    else if (turn == 2) {
                        if (moveToX == 0) {
                            this.selectedPiece.piece.isKing = true;
                        }
                    }
                    
                    // move the piece on the board
                    this.board.board[moveToX][moveToY].piece = this.board.board[this.selectedPiece.x][this.selectedPiece.y].piece;
                    this.board.board[this.selectedPiece.x][this.selectedPiece.y].piece = null;
                    this.board.board[this.selectedPiece.x][this.selectedPiece.y].hasPiece = false;
                    this.board.board[moveToX][moveToY].hasPiece = true;
                    
                    // eliminate the opponent piece
                    this.board.board[oppX][oppY].piece = null;
                    this.board.board[oppX][oppY].hasPiece = false;
                    
                    // check for multi-jumps
                    var jumps = this.getJumps(this.board.board[moveToX][moveToY], turn);
                    
                    
                    if (jumps.length > 0) {
                        // the user must select the next jump
                        this.ignoreInput = true; 
                        
                        // redraw the canvas
                        this.resetCanvas(this.context); 
                        
                        // draw the double jump
                        this.board.drawJumps(jumps, this.context); 
                        this.jumps = jumps;
                        this.selectedPiece = this.board.board[moveToX][moveToY];
                    }
                    else {
                        // allow regular input
                        this.ignoreInput = false;
                        
                        // redraw the canvas
                        this.resetCanvas(this.context);
                        
                        // un-select the piece
                        this.selectedPiece = null;
                        this.jumps = [];
                        
                        // get the element that displays whose turn it is
                        var turnDisplay = document.getElementById("turn");
                        // switch the turn
                        if (this.turn == 1) {
                            turnDisplay.innerHTML = "<h3>It is "+this.player2.name+"'s turn.</h3>";
                            this.turn = 2;
                        }
                        else {
                            turnDisplay.innerHTML = "<h3>It is "+this.player1.name+"'s turn.</h3>";
                            this.turn = 1;
                        }
                    } 
                    /* THIS IS NOT WORKING
                    // check for a stalemate
                    var stale = false;
                    if (turn == 1) {
                        stale = this.checkStale(this.player2);
                        if (stale == true) {
                            alert(this.player1.name+" has won!");
                            this.player1.numWins++;
                            saveGame(this.player1, this.player2);
                            
                            // restart the game
                            init();
                        }
                    }
                    else if (turn == 2) {
                        stale = this.checkStale(this.player1);
                        if (stale == true) {
                            alert(this.player2.name+" has won!");
                            this.player2.numWins++;
                            saveGame(this.player1, this.player2);
                            
                            // restart the game
                            init();
                        }
                    }  */  
                                    
                }
            }
            else {
                this.resetCanvas();
            }
            this.moves = [];
            
            // check if a player has won the game
            
            if (this.player1.numPieces == 0) {
                alert(this.player2.name+" has won!");
                this.player2.numWins++;
                saveGame(this.player1, this.player2);
                
                // restart the game
                init();
                
            }
            else if (this.player2.numPieces == 0) {
                alert(this.player1.name+" has won!");
                this.player1.numWins++;
                saveGame(this.player1, this.player2);
                
                // restart the game
                init();
            }
        }   
        else {
            this.resetCanvas();     
        } 
    }
}

// find all possible moves
GameManager.prototype.getMoves = function(piece, turn) {
    // array of spaces to move to
    var moveSpaces = new Array();
    
    if (turn == 1 && piece.piece.isKing == false) {
        // player 1's turn -- pieces can only move down
        if ((piece.x+1) < 8 && (piece.y-1) >= 0) {
            if (this.board.board[piece.x+1][piece.y-1].hasPiece == false) {
                moveSpaces.push({x: piece.x+1, y: piece.y-1});
            }
        }
        if ((piece.x+1) < 8 && (piece.y+1) < 8) {
            if (this.board.board[piece.x+1][piece.y+1].hasPiece == false) {
                moveSpaces.push({x: piece.x+1, y: piece.y+1});
            }
        }
    
    }
    else if (turn == 2 && piece.piece.isKing == false) {
        // player 2's turn -- pieces can only move up
        if (piece.x-1 >= 0 && piece.y-1 >= 0) {
            if (this.board.board[piece.x-1][piece.y-1].hasPiece == false) {
                moveSpaces.push({x: piece.x-1, y: piece.y-1});
            }
        }
        if (piece.x-1 >= 0 && piece.y+1 < 8) {
            if (this.board.board[piece.x-1][piece.y+1].hasPiece == false) {
                moveSpaces.push({x: piece.x-1, y: piece.y+1});
            }
        }
        
    }
    else if (piece.piece.isKing == true) {
        // piece is a king -- can move in either direction
        if ((piece.x-1) >= 0) {
            if (piece.y+1 < 8) {
                if (this.board.board[piece.x-1][piece.y+1].hasPiece == false) {
                    moveSpaces.push({x: piece.x-1, y: piece.y+1});
                }
            }
            if (piece.y-1 >= 0) {
                if (this.board.board[piece.x-1][piece.y-1].hasPiece == false) {
                    moveSpaces.push({x: piece.x-1, y: piece.y-1});
                }
            }
        }
        if ((piece.x+1) < 8) {
            if (piece.y+1 < 8) {
                if (this.board.board[piece.x+1][piece.y+1].hasPiece == false) {
                    moveSpaces.push({x: piece.x+1, y: piece.y+1});
                }
            }
            if (piece.y-1 >= 0) {
                if (this.board.board[piece.x+1][piece.y-1].hasPiece == false) {
                    moveSpaces.push({x: piece.x+1, y: piece.y-1});
                }
            }
        }        
    }
    // return all possible moves
    return moveSpaces;
}

// find all possible jumps
GameManager.prototype.getJumps = function(piece, turn) {
    // array of spaces to jump to
    var jumpSpaces = new Array();
    
    // board
    var board = this.board.board;
    
    if (turn == 1 && piece.piece.isKing == false) {
        // player 1's turn -- pieces can only move down
        if ((piece.y-1) >= 0 && (piece.y-2) >= 0 && (piece.x+1) < 8 && (piece.x+2) < 8) {
            if (board[piece.x+1][piece.y-1].hasPiece == true && board[piece.x+1][piece.y-1].piece.colour !== "W") {
                if (board[piece.x+2][piece.y-2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x+2, y: piece.y-2, oppX: piece.x+1, oppY: piece.y-1});
                }
            }
        }
        if ((piece.y+1) < 8 && (piece.y+2) < 8 && (piece.x+1) < 8 && (piece.x+2) < 8) {
            if (board[piece.x+1][piece.y+1].hasPiece == true && board[piece.x+1][piece.y+1].piece.colour !== "W") {
                if (board[piece.x+2][piece.y+2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x+2, y: piece.y+2, oppX: piece.x+1, oppY: piece.y+1});
                }
            }
        }
    }
    else if (turn == 2 && piece.piece.isKing == false) {
        // player 2's turn -- pieces can only move up
        if (piece.y-1 >= 0 && (piece.y-2) >= 0 && (piece.x-1) >= 0 && (piece.x-2) >= 0) {
            if (board[piece.x-1][piece.y-1].hasPiece == true && board[piece.x-1][piece.y-1].piece.colour !== "B") {
                if (board[piece.x-2][piece.y-2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x-2, y: piece.y-2, oppX: piece.x-1, oppY: piece.y-1});
                }
            }
        }
        if (piece.y+1 < 8 && (piece.y+2) < 8 && (piece.x-1) >= 0 && (piece.x-2) >= 0) {
            if (board[piece.x-1][piece.y+1].hasPiece == true && board[piece.x-1][piece.y+1].piece.colour !== "B") {
                if (board[piece.x-2][piece.y+2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x-2, y: piece.y+2, oppX: piece.x-1, oppY: piece.y+1});
                }
            }
        }
    }
    else if (piece.piece.isKing == true) {
        // kings can move in either direction
        if (piece.y-1 >= 0 && (piece.y-2) >= 0 && (piece.x-1) >= 0 && (piece.x-2) >= 0) {
            if (board[piece.x-1][piece.y-1].hasPiece == true && board[piece.x-1][piece.y-1].piece.colour !== piece.piece.colour) {
                if (board[piece.x-2][piece.y-2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x-2, y: piece.y-2, oppX: piece.x-1, oppY: piece.y-1});
                }
            }
        }
        if ((piece.y-1) >= 0 && (piece.y-2) >= 0 && (piece.x+1) < 8 && (piece.x+2) < 8) {
            if (board[piece.x+1][piece.y-1].hasPiece == true && board[piece.x+1][piece.y-1].piece.colour !== piece.piece.colour) {
                if (board[piece.x+2][piece.y-2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x+2, y: piece.y-2, oppX: piece.x+1, oppY: piece.y-1});
                }
            }
        }
        if (piece.y+1 < 8 && (piece.y+2) < 8 && (piece.x-1) >= 0 && (piece.x-2) >= 0) {
            if (board[piece.x-1][piece.y+1].hasPiece == true && board[piece.x-1][piece.y+1].piece.colour !== piece.piece.colour) {
                if (board[piece.x-2][piece.y+2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x-2, y: piece.y+2, oppX: piece.x-1, oppY: piece.y+1});
                }
            }
        }
        if ((piece.y+1) < 8 && (piece.y+2) < 8 && (piece.x+1) < 8 && (piece.x+2) < 8) {
            if (board[piece.x+1][piece.y+1].hasPiece == true && board[piece.x+1][piece.y+1].piece.colour !== piece.piece.colour) {
                if (board[piece.x+2][piece.y+2].hasPiece == false) {
                    jumpSpaces.push({x: piece.x+2, y: piece.y+2, oppX: piece.x+1, oppY: piece.y+1});
                }
            }
        }        
    }
    // return all possible moves
    return jumpSpaces;
}

// reset the canvas screen
GameManager.prototype.resetCanvas = function() {
    this.board.draw(this.context);
}

// write score to screen
GameManager.prototype.writeScore = function() {
    // Write the scoreboard to the left table
    var table = document.getElementById("players");
    table.innerHTML = "<tr><th>Player</th><th># of Wins</th></tr>";
    
    var tr1 = document.createElement("tr");
    tr1.innerHTML = "<td>"+this.player1.name+"</td><td>"+this.player1.numWins+"</td>";
    
    var tr2 = document.createElement("tr");
    tr2.innerHTML = "<td>"+this.player2.name+"</td><td>"+this.player2.numWins+"</td>";
    
    table.appendChild(tr1);
    table.appendChild(tr2);
}
/* THIS IS NOT WORKING PROPERLY
// Check if a player cannot move
GameManager.prototype.checkStale = function(player) {
    var validMoves = new Array();
    
    // check each piece if there is a valid move
    for (var i = 0; i < 7; i++) {
        for (var j = 0; j < 7; j++) {
            square = this.board.board[i][j];
            if (square.hasPiece == true) {
                if (square.piece.colour == player.colour) {
                    var moves = this.getMoves(square, this.turn);
                    var jumps = this.getJumps(square, this.turn);
                    if (moves.length > 0) {
                        validMoves.push(moves);
                    }
                    if (jumps.length > 0) {
                        validMoves.push(jumps);
                    }
                }
            }
        }
    }
    // return false if there are possible moves
    if (validMoves.length > 0) {
        return false;
    }
    return true;
}*/
