/************************************
    Board, Piece and Square Object
************************************/

var Square = function(hasPiece, piece, x, y, width) {
    
    // set square object members
    this.hasPiece = hasPiece;
    this.piece = piece;
    this.x = x;
    this.y = y;
    this.width = width;
}
    
// Check to see if a piece is selected
Square.prototype.isSelected = function(x, y) {
    // check if the selection falls within the area of the circle
    return (((x <= this.x*this.width+this.width) && (x >= this.x*this.width)) && ((y <= this.y*this.width+this.width) && (y >= this.y*this.width)));
}

var Piece = function(colour, x, y, rad) {

    // set member variables of piece
    this.colour = colour;
    this.x = x;
    this.y = y;
    this.rad = rad;
    this.isKing = false;
    
    // set the colour of the piece
    if (this.colour == "W") {
        this.fill = "#FFFFFF";
    }
    else {
        this.fill = "#000000";
    }
    
    // print the colour of the piece out to screen
    this.toString = function() {
        return this.colour;
    }
}

    
// Draw a piece to the screen
Piece.prototype.draw = function(context) {
    context.fillStyle = this.fill;
    context.beginPath();
    context.arc(this.x,this.y,this.rad,0, Math.PI*2, true);
    context.closePath();
    context.fill();
    
    // draw a "K" on the piece of it is a king
    if (this.isKing == true) {
        if (this.colour == "W") {
            context.strokeStyle = "rgba(0,0,0,1)";
        }
        else {
            context.strokeStyle = "rgba(255,255,255,1)";
        }
        
        // draw a line
        context.lineWidth = 3;
        context.beginPath();
        context.moveTo(this.x-8, this.y-this.rad+5);
        context.lineTo(this.x-8, this.y+this.rad-5);
        context.stroke();
        context.closePath();
        
        // draw two more lines
        context.beginPath();
        context.moveTo(this.x-8, this.y+(this.rad/2)-9);
        context.lineTo(this.x+(this.rad-10), this.y-this.rad+6);
        context.stroke();
        context.closePath();
        context.beginPath();
        context.moveTo(this.x-8, this.y+(this.rad/2)-9);
        context.lineTo(this.x+(this.rad-10), this.y+this.rad-6);
        context.stroke();
        context.closePath();
    }
}

var Board = function(width) {

    // set the width of a piece of the board
    this.spaceWidth = width;
    
    // create a two-dimensional board -- will be either W, B, or _
    this.board = new Array(8);
    
    // go two-dimensional
    for (var i = 0; i < 8; i++) {
        this.board[i] = new Array(8);
    }
    
    // Set the board to clear
    for (var i = 0; i < 8; i++) {
        for (var j =0; j < 8; j++) {
            this.board[i][j] = new Square(false, null, i, j, this.spaceWidth);
        }
    }
    
    // Create the initial board
    for (var i = 0; i < 2; i++) {
        // determine where to place the first place
        var firstPiece = 0;
        if (i == 0) {
            var alt = 1;
            var row = -1;
        }
        else {
            var alt = -1;
            var row = 4;
        }
        for (var j = 0; j < 12; j++) {
            // set the piece colour
            if (i == 0) {
                var newStr = "W";
            }
            else {
                var newStr = "B";
            }
            
            // move the row down
            if (j % 4 == 0) {
                row++;    
                if (alt == 1) {
                    firstPiece = 1;                
                }            
                else {
                    firstPiece = 0;
                }
                alt *= -1;
            }
            
            // create the piece
            this.board[row][((j%4)*2)+firstPiece] = new Square(true, new Piece(newStr, 2*(j%4)*this.spaceWidth+((this.spaceWidth/2)+firstPiece*this.spaceWidth), 
                                                                (row*this.spaceWidth)+(this.spaceWidth/2), (this.spaceWidth/2)-10), row, ((j%4)*2)+firstPiece, this.spaceWidth);
        }
    }
    
    // overload toString function
    this.toString = function() {
        var boardString = "<table>";
        for (var i = 0; i < 8; i++) {
            boardString += "<tr>"
            for (var j = 0; j < 8; j++) {
                boardString += "<td>"+this.board[i][j]+"</td>";
            }
            boardString += "</tr>";
        }
        boardString +="</table>";
        return boardString;
    }

}

// draw the board to the screen
Board.prototype.draw = function (context) {
    // some variables for drawing
    var row = -1*this.spaceWidth;
    var alt = 1;
           
    // fill in a checkerboard
    for (var i = 0; i < 64; i++) {
        // increment the row each 8 spots
        if (i % 8 == 0) {
            row += this.spaceWidth;
            if (i == 0) {
                alt *= -1;
            }
        }
        else {
            alt *= -1;
        }
                
        // select the correct colour
        if (alt === 1) {
            context.fillStyle = "rgba(180,0,0,1)";
        }
        else if (alt === -1) {
            context.fillStyle = "rgba(0,0,0,1)";
        }
              
        // draw the rectangle
        context.fillRect((i%8)*this.spaceWidth, row, this.spaceWidth, this.spaceWidth);        
    }
    
    // draw each piece
    for (var i = 0; i < 8; i++) {
        for (var j = 0; j < 8; j++) {
            // check if there is a piece
            if (this.board[i][j].hasPiece == true) {
                // draw the piece
                this.board[i][j].piece.draw(context);
            }
        }
    }
}

// highlight the moves on the board
Board.prototype.drawMoves = function(moves, context) {
    // draw a yellow square on each move space
    for (var i = 0; i < moves.length; i++) {  
        context.beginPath();
        context.strokeStyle = "rgba(255,255,0,1)";
        context.lineWidth = 3;
        context.rect(this.spaceWidth*moves[i].y, this.spaceWidth*moves[i].x, this.spaceWidth-1.5, this.spaceWidth-1.5);
        context.stroke();
        context.closePath();
    }
}

// highlight the jumps on the board
Board.prototype.drawJumps = function(jumps, context) {
    // draw a blue square on each jump space       
    for (var i = 0; i < jumps.length; i++) {  
        context.beginPath();
        context.strokeStyle = "rgba(0,0,255,1)";
        context.lineWidth = 3;
        context.rect(this.spaceWidth*jumps[i].y, this.spaceWidth*jumps[i].x, this.spaceWidth-1.5, this.spaceWidth-1.5);
        context.stroke();
        context.closePath();
    }
}
