/* Game Manager Object */

const STATE_INGAME = "ingame";
const STATE_NOT_INGAME = "notingame";

var game = new gameManager();

function gameManager()
{
    
    this.players = [];
    this.renderer = null;
    this.board = null;
    this.transpose = null;
    this.gameState = null;
    this.storage = null;

    this.currentPlayer = null;

    this.canvasElem = null;
    this.context = null;

}


gameManager.prototype.startGame = function () {

    this.board.reset();
    this.transpose.reset();

    this.renderer.drawBoard(this.board.boardSize);

    this.renderer.drawTurn(this.currentPlayer);

    this.gameState = STATE_INGAME;

} 

gameManager.prototype.nextPlayer = function () {
    if (this.currentPlayer == "white") {
        this.currentPlayer = "black";
    }
    else {
        this.currentPlayer = "white";
    }
}

gameManager.prototype.init = function (game) {

 
    this.storage = new Storage();

    this.board = new Board(15,"empty");
    this.transpose = new Board(15, "empty");
  
    this.drawGameObjects();

    this.canvasElem = document.getElementById("boardCanvas");
    this.context = this.canvasElem.getContext("2d");

    this.renderer = new Renderer(this.context, this.canvasElem.width,this.board.boardSize);

    this.login();

    this.currentPlayer = "black";

    this.canvasElem.addEventListener('click', function(event)
    {  

        if (game.gameState === STATE_INGAME) {

            var x = event.pageX - this.offsetLeft;
            var y = event.pageY - this.offsetTop;

    	    var xcell = Math.floor((x / game.renderer.cellWidth)+0.5)-1; // round to
	        var ycell = Math.floor((y / game.renderer.cellWidth)+0.5)-1; // nearest integer
    
            if (xcell >= 0 && xcell < 15 && ycell >= 0 && ycell < 15 
                && game.board.data[xcell][ycell] === game.board.defaultCellValue) {
    
                game.renderer.drawStone(xcell+1,ycell+1,game.currentPlayer);

                game.board.data[xcell][ycell] = game.currentPlayer;
                game.transpose.data[game.transpose.boardSize-1-xcell][ycell] = game.currentPlayer;

                if (game.board.checkWin(xcell,ycell) == true || game.transpose.checkWin(game.transpose.boardSize-1-xcell,ycell) == true ) {
                    if (game.currentPlayer === "black") {
                        alert(game.players[0].playerName+" WINS!!!");
                        game.players[0].numWins++;
                        game.players[1].numLosses++;
                    }
                    else {
                        alert(game.players[1].playerName+" WINS!!!");
                        game.players[1].numWins++;
                        game.players[0].numLosses++;
                    }
                    game.storage.savePlayerStats(game.players[0],game.players[1]);
                    game.renderer.drawRecordBoard(game.players[0],game.players[1]);
                    game.gameState = STATE_NOT_INGAME;
                }
                game.nextPlayer();
                game.renderer.drawTurn(game.currentPlayer);
            }
        }        
    }, false);

    this.renderer.drawOptions(game);
    this.renderer.drawRecordBoard(this.players[0],this.players[1]);


    this.startGame();

}

gameManager.prototype.drawGameObjects = function () {
    var gameBoard = document.getElementById("gameBoard");
    gameBoard.innerHTML = (
            "<canvas id='boardCanvas' width='510' height='510'>"+ 
            "</canvas>")
}

gameManager.prototype.login = function ()
{
    var playerNames = [];
    playerNames[0] = "";
    playerNames[1] = "";
    for (var i=0;i<2;i++) 
    {
        while(playerNames[i] === "") {
            playerNames[i] = prompt("Please enter the name of Player "+(i+1));
        }
    }
    this.players[0] = this.storage.loadPlayerStats(playerNames[0]);
    this.players[1] = this.storage.loadPlayerStats(playerNames[1]);
    if (this.players[0] === null) {
        this.players[0] = new Player(playerNames[0]);
    }
    if (this.players[1] === null) {
        this.players[1] = new Player(playerNames[1]);
    }
    this.storage.savePlayerStats(this.players[0],this.players[1]);
    this.renderer.drawRecordBoard(this.players[0],this.players[1]);
}

gameManager.prototype.loginButton = function ()
{
    if (this.gameState === STATE_INGAME) {
        alert("ERROR: Currently in game! Please finish before switching players!");
        return;
    }
    else {
        this.login();
    }
}

gameManager.prototype.startButton = function ()
{
    if (this.gameState === STATE_INGAME) {
        alert("ERROR: Currently in game! Please finish before starting a new one!");
        return;
    }
    else {
        this.startGame();
    }
}

gameManager.prototype.resignButton = function ()
{
    if (this.gameState != STATE_INGAME) {
        alert("ERROR: You can't resign because you are not in a game! Start a new game by clicking the 'Start Game!' button.");
        return;
    }
    else {
        if (game.currentPlayer === "black") {
            alert(this.players[0].playerName+" Resigns!!! Therefore "+this.players[1].playerName+" Wins!!!");
            this.players[1].numWins++;
            this.players[0].numLosses++;
        }
        else {
            alert(this.players[1].playerName+" Resigns!!! Therefore "+this.players[0].playerName+" Wins!!!");
            this.players[0].numWins++;
            this.players[1].numLosses++;
       }
       this.storage.savePlayerStats(game.players[0],game.players[1]);
       this.renderer.drawRecordBoard(game.players[0],game.players[1]);
       this.gameState = STATE_NOT_INGAME;
    }
}


function init()
{
    game.init(game);
}
