
function Ball()
{
	this.x = 0; // Horizontal position percentage (actual = this.x/canvas.width)
	this.y = 0; // Vertical   position percentage   (actual = this.y/canvas.height)
	this.r = 0; // radius in pixels
	
	this.s = 0; // speed of ball in percentage of max speed (actual = this.s*this.speed
	this.d = 0; // direction of ball in radians
	
	this.maxs = 0.019; // Max speed in percentage (Max area that can be covered in one update)
	this.mins = 0.12; // minimum this.s can be
	this.sg = 0.06; // speed growth when hitting a paddle
	this.sd = 0.006; // speed decay when hitting a wall
	
	
	this.cx = [0,0]; //anything in this range is a collision [left,right]
	this.cy = [0,0]; //anything in this range is a collision [bottom, top]
	
	this.setCX = function(canvas)
	{
		this.cx = [ ((this.x)-(this.r/canvas.width)) , ((this.x)+(this.r/canvas.width)) ];
	};
	
	this.setCY = function(canvas)
	{
		this.cy = [ ((this.y)-(this.r/canvas.height)) , ((this.y)+(this.r/canvas.height)) ];
	};
	
	this.setC = function(canvas)
	{
		this.setCX(canvas);
		this.setCY(canvas);
	};
	
	
	this.draw = function(canvas)
	{
		var context = canvas.getContext("2d");
		
		//Draw ball
		context.beginPath();
		context.arc(this.x*canvas.width, this.y*canvas.height, this.r, 0, 2 * Math.PI);
		context.fillStyle = 'white';
		context.fill();
	};
	
	this.hasCollidedWith = function(paddle, canvas)
	{
		var width = canvas.width;
		var height = canvas.height;
		
		pX = paddle.centerX*width;
		pY = paddle.centerY*height;
		bX = this.x*width;
		bY = this.y*height;
		
		var circleDistanceX = Math.abs(bX - pX);
		var circleDistanceY = Math.abs(bY - pY);
		
		if (circleDistanceX > (paddle.w/2 + this.r)) { return false; }
		if (circleDistanceY > (paddle.h/2 + this.r)) { return false; }
		
		if (circleDistanceX <= (paddle.w/2)) { return true; }
		if (circleDistanceY <= (paddle.h/2)) { return true; }
		
		var cornerDistance = Math.pow((circleDistanceX - paddle.w/2), 2) + Math.pow((circleDistanceY - paddle.h/2), 2);
		
		return cornerDistance <= Math.pow(this.r, 2);
	};
	
	this.update = function(canvas)
	{	
		//uInterval = window.clearInterval(uInterval);
		
		var context = canvas.getContext("2d");
		var width = canvas.width;
		var height = canvas.height;
		
		// radius to percent
		var rx = this.r/width;
		var ry = this.r/height;
		
		// old x and y
		var oldx = this.x;
		var oldy = this.y;
		
		// speed
		var speed = this.s;
		var maxspeed = this.maxs;
		
		// actual speed
		var aspeed = speed*maxspeed;
		
		// direction
		var direction = this.d;
		
		
		// determine new position
		var newx = (Math.cos(direction) * aspeed) + oldx;
		var newy = (Math.sin(direction) * aspeed) + oldy;
		
		// determine new collision bounds
		var newcx = [ newx-rx , newx+rx ];
		var newcy = [ newy-ry , newy+ry ];
		
		this.d = direction;
		this.x = newx;
		this.y = newy;
		this.cx = newcx;
		this.cy = newcy;
		this.s = speed;
	};//*END update()/**/
	
	this.collisions = function (canvas)
	{
		
		var context = canvas.getContext("2d");
		var width = canvas.width;
		var height = canvas.height;
		
		// radius to percent
		var rx = this.r/width;
		var ry = this.r/height;
		
		// speed
		var speed = this.s;
		var maxspeed = this.maxs;
		var minspeed = this.mins;
		var growth = this.sg;
		var decay = this.sd;
		
		// actual speed
		var aspeed = speed*maxspeed;
		
		// direction
		var direction = this.d;
		var olddirection = this.d;
		
		// new position
		var newx = this.x;
		var newy = this.y
		
		// new collision bounds
		var newcx = this.cx;
		var newcy = this.cy;
		
		var paddles = game.paddles;
		
		
		//HITS A PADDLE
		for (var i = 0; i < paddles.length; i++)
		{
			
			var paddle = paddles[i];
			var pcx = paddle.cx;
			var pcy = paddle.cy;
			
			if (this.hasCollidedWith(paddle, canvas))
			{
				//uInterval = window.clearInterval(uInterval);
				
				var xDistance = 0;
				var yDistance = 0;
				var tDistance = 0;
				
				var vOverlap = 0;
				var hOverlap = 0;
				
				if (direction > 0 && direction < Math.PI)
				{// Ball is going up
				
					vOverlap = (newcy[1] - pcy[0]) * height;
				
					if (direction < Math.PI/2)
					{// Ball is going right
					
						hOverlap = (newcx[1] - pcx[0]) * width;
						
						if (vOverlap < hOverlap)
						{// Ball collided with the bottom of the paddle
							newy = pcy[0] - ry;
							
							yDistance = vOverlap/height;
							xDistance = Math.tan(direction) * yDistance;
							if ((newx - pcx[0] - xDistance) >= 0 && (pcx[1] - newx - xDistance) >= 0)
							{// Face collision
								if (pcx[1] - newcx[1] - xDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2)) + (Math.PI/2 - direction)/3;
								}
								
								else if (newcx[0] - pcx[0] - xDistance < 0)
								{// Decresing
									direction = (-direction % (Math.PI*2)) - (Math.PI/2 - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2));
								}
							}
							else if ((pcx[1] - newx - xDistance) < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2)) + (Math.PI/2 - direction)/2;
							}
							else
							{// Decreasing corner collision
								direction = (-direction % (Math.PI*2)) - (Math.PI/2 - direction)/2;
							}
							
						}
						else
						{// Ball collided with the left side of the paddle
							newx = pcx[0] - rx;
							
							xDistance = hOverlap/width;
							yDistance = xDistance / Math.tan(direction);
							if ((newy - pcy[0] - yDistance) >= 0 && (pcy[1] - newy - yDistance) >= 0)
							{// Face Collision
								if (pcy[1] - newcy[1] - yDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI) - (Math.PI/2 - direction)/3;
								}
								
								else if (newcy[0] - pcy[0] - yDistance < 0)
								{// Decreasing
									direction = (-direction % (Math.PI*2) + Math.PI) + (Math.PI/2 - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI);
								}
							}
							else if ((pcy[1] - newy - yDistance) < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2) + Math.PI) - (Math.PI/2 - direction)/2;
							}
							else
							{// Decreasing corner collision
								direction = (-direction % (Math.PI*2) + Math.PI) + (Math.PI/2 - direction)/2;
							}
							
						}
						
					}
					else if (direction > Math.PI/2)
					{// Ball is going left
						
						hOverlap = (pcx[1] - newcx[0]) * width;
						
						if (vOverlap < hOverlap)
						{// Ball collided with the bottom of the paddle
	
							newy = pcy[0] - ry;
							
							yDistance = vOverlap/height;
							xDistance = Math.tan(direction) * yDistance;
							if ((newx - pcx[0] - xDistance) >= 0 && (pcx[1] - newx - xDistance) >= 0)
							{// Face collision
								
								if (newcx[0] - pcx[0] - xDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2)) - (Math.PI - direction)/3;
								}
								
								else if (pcx[1] - newcx[1] - xDistance < 0)
								{// Decreasing
									direction = (-direction % (Math.PI*2)) + (Math.PI - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2));
								}
							}
							else if (newx - pcx[0] - xDistance < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2)) - (Math.PI - direction)/2;
							}
							else
							{// Decreasing corner collision
								direction = (-direction % (Math.PI*2)) + (Math.PI - direction)/2;
							}							
						}
						else
						{// Ball collided with the right side of the paddle						
							newx = pcx[1] + rx;
							
							xDistance = hOverlap/width;
							yDistance = xDistance / Math.tan(direction);
							if ((newy - pcy[0] - yDistance) >= 0 && (pcy[1] - newy - yDistance) >= 0)
							{// Face collision
								
								if (newcy[0] - pcy[0] - yDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI) + (Math.PI - direction)/3;
								}
								
								else if (pcy[1] - newcy[1] - yDistance < 0)
								{// Decreasing
									direction = (-direction % (Math.PI*2) + Math.PI) - (Math.PI - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI);
								}
							}
							else if ((pcy[1] - newy - yDistance) < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2) + Math.PI) + (Math.PI - direction)/2;
							}
							else
							{// Decresing corner collision
								direction = (-direction % (Math.PI*2) + Math.PI) - (Math.PI - direction)/2;
							}
							
						}
						
					}
					else
					{// Ball is going straight up
						newy = pcy[0] - ry;
					
						if ((newx - pcx[0]) >= 0 && (pcx[1] - newx) >= 0)
						{// Face collision
							if (pcx[1] - newcx[1] < 0)
							{// right side
								direction = direction + (Math.PI/6) + Math.PI;
							}
							else if (newcx[0] - pcx[0] < 0)
							{// left side
								direction = direction - (Math.PI/6) + Math.PI;
							}
							else
							{// Normal
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2));
							}
						}
						else if ((pcx[1] - newx) < 0)
						{// right corner
							direction = direction + (Math.PI/4) + Math.PI;
						}
						else
						{// left corner
							direction = direction - (Math.PI/4) + Math.PI;
						}
					}
				}
				else if (direction > Math.PI)
				{// Ball is going down
				
					vOverlap = (pcy[1] - newcy[0]) * height;
				
					if (direction > (3*Math.PI)/2)
					{// Ball is going right
					
						hOverlap = (newcx[1] - pcx[0]) * width;
						
						if (vOverlap < hOverlap)
						{// Ball collided witht the top of the paddle
							newy = pcy[1] + ry;
							
							yDistance = vOverlap/height;
							xDistance = Math.tan(direction) * yDistance;
							if ((newx - pcx[0] - xDistance) >= 0 && (pcx[1] - newx - xDistance) >= 0)
							{// Face collision
								
								if (pcx[1] - newcx[1] - xDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2)) - (Math.PI*2 - direction)/3;
								}
								
								else if (newcx[0] - pcx[0] - xDistance < 0)
								{// Decreasing
									direction = (-direction % (Math.PI*2)) + (Math.PI*2 - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2));
								}
							}
							else if ((pcx[1] - newx - xDistance) < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2)) - (Math.PI*2 - direction)/2;
							}
							else
							{// Decresing corner collision
								direction = (-direction % (Math.PI*2)) + (Math.PI*2 - direction)/2;
							}
							
						}
						else
						{// Ball collided with the left side of the paddle
							newx = pcx[0] - rx;
							
							xDistance = hOverlap/width;
							yDistance = xDistance / Math.tan(direction);
							if ((newy - pcy[0] - yDistance) >= 0 && (pcy[1] - newy - yDistance) >= 0)
							{// Face collision
								
								if (newcy[0] - pcy[0] - yDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI) + (Math.PI*2 - direction)/3;
								}
								
								else if (pcy[1] - newcy[1] - yDistance < 0)
								{// Decreasing
									direction = (-direction % (Math.PI*2) + Math.PI) - (Math.PI*2 - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI);
								}
							}
							else if ((newy - pcy[0] - yDistance) < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2) + Math.PI) + (Math.PI*2 - direction)/2;
							}
							else
							{// Decresing corner collision
								direction = (-direction % (Math.PI*2) + Math.PI) - (Math.PI*2 - direction)/2;
							}
							
						}
						
					}
					else if (direction < (3*Math.PI)/2)
					{// Ball is going left
					
						hOverlap = (pcx[1] - newcx[0]) * width;
						
						if (vOverlap < hOverlap)
						{// Ball collided with the top of the paddle
							newy = pcy[1] + ry;
							
							yDistance = vOverlap/height;
							xDistance = Math.tan(direction) * yDistance;
							if ((newx - pcx[0] - xDistance) >= 0 && (pcx[1] - newx - xDistance) >= 0)
							{// Face collision
								
								if (newcx[0] - pcx[0] - xDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2)) + ((Math.PI*3)/2 - direction)/3;
								}
								
								else if (pcx[1] - newcx[1] - xDistance < 0)
								{// Decreasing
									direction = (-direction % (Math.PI*2)) - ((Math.PI*3)/2 - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2));
								}
							}
							else if (newx - pcx[0] - xDistance < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2)) + ((Math.PI*3)/2 - direction)/2;
							}
							else
							{// Decresing corner collision
								direction = (-direction % (Math.PI*2)) - ((Math.PI*3)/2 - direction)/2;
							}
							
						}
						else
						{// Ball collided with the right side of the paddle
							newx = pcx[1] + rx;
							
							xDistance = hOverlap/width;
							yDistance = xDistance / Math.tan(direction);
							if ((newy - pcy[0] - yDistance) >= 0 && (pcy[1] - newy - yDistance) >= 0)
							{// Face collision
								
								if (pcy[1] - newcy[1] - yDistance < 0)
								{// Increasing
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI) - ((Math.PI*3)/2 - direction)/3;
								}
								
								else if (newcy[0] - pcy[0] - yDistance < 0)
								{// Decreasing
									direction = (-direction % (Math.PI*2) + Math.PI) + ((Math.PI*3)/2 - direction)/3;
								}
								else
								{// Normal
									if (speed+growth < 1)
									{
										speed += growth;
									}
									else
									{
										speed = 1;
									}
									direction = (-direction % (Math.PI*2) + Math.PI);
								}
							}
							else if ((newy - pcy[0] - yDistance) < 0)
							{// Increasing corner collision
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2) + Math.PI) - ((Math.PI*3)/2 - direction)/2;
							}
							else
							{// Decresing corner collision
								direction = (-direction % (Math.PI*2) + Math.PI) + ((Math.PI*3)/2 - direction)/2;
							}
							
						}
						
					}
					else
					{// Ball is going straight down
						newy = pcy[1] + ry;
					
						if ((newx - pcx[0]) >= 0 && (pcx[1] - newx) >= 0)
						{// Face collision
							if (newcx[0] - pcx[0] < 0)
							{// right side
								direction = direction + (Math.PI/6) + Math.PI;
							}
							else if (pcx[1] - newcx[1] < 0)
							{// left side
								direction = direction - (Math.PI/6) + Math.PI;
							}
							else
							{// Normal
								if (speed+growth < 1)
								{
									speed += growth;
								}
								else
								{
									speed = 1;
								}
								direction = (-direction % (Math.PI*2));
							}
						}
						else if (newx - pcx[0] < 0)
						{// right corner
							direction = direction + (Math.PI/4) + Math.PI;
						}
						else
						{// left corner
							direction = direction - (Math.PI/4) + Math.PI;
						}					
					}
				}
				else
				{// Ball is going straight across
				
					//will never happen
					game.clean();
					quit();
					alert('ERROR: Ball straight across');
					
				}
			}
			
		}
		//END HITS A PADDLE
		
		
		//HITS A WALL
		if (newcy[0] < 0 || newcy[1] > 1)
		{	
			if (newcy[0] <= 0)
			{
				game.score(1);
				return;
			}
			else
			{
				game.score(0);
				return;
			}
		}
		
		if (newcx[0] < 0 || newcx[1] > 1)
		{
			if (newcx[0] <= 0)
			{
				newx = 0 + rx;
				newcx = [ 0 , newx+rx ];
			}
			else
			{
				newx = 1 - rx;
				newcx = [ newx-rx , 1 ];
			}
			
			direction = (-direction % (Math.PI*2) + Math.PI);
		}
		//END HITS A WALL
		
		if (direction < 0)
		{
			direction += Math.PI*2;
		}
		
		this.d = direction;
		this.x = newx;
		this.y = newy;
		this.cx = newcx;
		this.cy = newcy;
		this.s = speed;
	}
	//*END update()
}
//**END Ball()