function init()
{
	var playButton = document.getElementById("play_button");
	var loginButton = document.getElementById("login_button");
	
	playButton.addEventListener("click", play, false);
	loginButton.addEventListener("click", login, false);

	loadLeaderboard();
}



//Leaderboard stuff
function loadLeaderboard()
{
	var leaderboard = JSON.parse(window.localStorage.getItem("leaderboard"));
	if (leaderboard)
	{
		displayLeaderboard(leaderboard);
	}
}
function saveLeaderboard(leaderboard) 
{
	window.localStorage.setItem('leaderboard', JSON.stringify(leaderboard));
}
function displayLeaderboard(leaderboard)
{
	leaderboard.sort(sortLeaderboard);
	
	var tbody = document.getElementById('scores');
	tbody.innerHTML = '';
	
    for (var i = 0; i < leaderboard.length; i++)
    {
        var entry = leaderboard[i];
        
        var row = document.createElement('tr');
        var name = document.createElement('td');
        name.innerHTML = entry['name'];
        var wins = document.createElement('td');
        wins.innerHTML = entry['wins'];
        var losses = document.createElement('td');
        losses.innerHTML = entry['losses'];
        var rank = document.createElement('td');
        rank.innerHTML = entry['rank'];
        
        row.appendChild(name);
        row.appendChild(wins);
        row.appendChild(losses);
        row.appendChild(rank);
        
        tbody.appendChild(row);
    }
}
function sortLeaderboard(a,b)
{	
	if (parseFloat(a['rank']) < parseFloat(b['rank']))
	{
		return 1;
	}
	else if (parseFloat(a['rank']) > parseFloat(b['rank']))
	{
		return -1;
	}
	else
	{
		return (parseFloat(b['rank']) - parseFloat(a['rank']));
	}
}
//end Leaderboard stuff


function play()
{
	var startSection = document.getElementById("start_section");
	var loginSection = document.getElementById("login_section");
	var loginForm = document.getElementById("login_form");
	
	startSection.style.display = 'none';
	loginSection.style.display = 'block';
	loginForm.addEventListener("keyup", loginFormKeyUp, true);
}
function login()
{
	var p1un = document.getElementById('p1_username_field').value;
	var p2un = document.getElementById('p2_username_field').value;
	
	if (p1un == '' || p2un == '')
	{
		alert("Both players must have a name");
		return;
	}
	if (p1un == p2un)
	{
		alert("Players cannot have the same name");
		return;
	}
	
	var loginSection = document.getElementById("login_section");
	var gameSection = document.getElementById("game_section");
	var loginForm = document.getElementById("login_form");
	
	loginSection.style.display = 'none';
	gameSection.style.display = 'block';
	
	loginForm.removeEventListener("keyup", loginFormKeyUp, true);
	
	game = new Game(p1un, p2un);
	
	var menuButtonContainers = document.getElementsByClassName("menu_buttons_container");
	for (var i = 0; i < menuButtonContainers.length; i++)
	{
		menuButtonContainers[i].childNodes[1].addEventListener("click", function(){game.setup();}, false);
		menuButtonContainers[i].childNodes[3].addEventListener("click", quit, false);
	}
	
	setTimeout(function(){game.setup();},50); //required so the keyup event that triggered login() doesn't trigger the game to start
}
function quit()
{
	game.clean();
	
	var menu = document.getElementById("menu");
	
	menu.style.display = 'none';
	
	var gameSection = document.getElementById("game_section");
	var startSection = document.getElementById("start_section");
	
	gameSection.style.display = 'none';
	startSection.style.display = 'block';
}




//Listeners
function loginFormKeyUp(e)
{
	var enterKey = 13;
		
	if (e.keyCode == enterKey)
	{
		login();
	}
}

function keyDownHandler(e)
{
	switch (e.keyCode)
	{
		case game.KEY_CODES['p1left']: game.paddles[1].left=true; game.paddles[1].right=false; break;
		case game.KEY_CODES['p1right']: game.paddles[1].right=true; game.paddles[1].left=false; break;
		case game.KEY_CODES['p2left']: game.paddles[0].left=true; game.paddles[0].right=false; break;
		case game.KEY_CODES['p2right']: game.paddles[0].right=true; game.paddles[0].left=false; break;
	}
}
function keyUpHandler(e)
{
	switch (e.keyCode)
	{
		case game.KEY_CODES['p1left']: game.paddles[1].left=false; break;
		case game.KEY_CODES['p1right']: game.paddles[1].right=false; break;
		case game.KEY_CODES['p2left']: game.paddles[0].left=false; break;
		case game.KEY_CODES['p2right']: game.paddles[0].right=false; break;
	}
}
function menuKeyListener(e)
{
	if (e.keyCode == game.KEY_CODES['escape'])
	{
		var menu = document.getElementById("menu");
		if (menu.style.display != 'block')
		{
			menu.style.display = 'block';
			game.stop();
		}
		else
		{
			menu.style.display = 'none';
			if (!game.isStart && !game.isScored)
			{
				game.start();
			}
		}
	}

}
function startKeyListener(e)
{
	if (e.keyCode == game.KEY_CODES['enter'])
	{
		var start = document.getElementById("start");
		var scored = document.getElementById("scored");
		
		start.style.display = 'none';
		scored.style.display = 'none';
		
		window.removeEventListener("keyup", startKeyListener);
		
		game.start();
	}

}