function gameBoard(){
	//x is rows
	//y is columns

	var BOARD_SIZE = 10;
	var MINE_COUNT = 21;
	//create board table
	this.board = new Array(BOARD_SIZE);
	for (var i = 0; i < BOARD_SIZE; i++){
		this.board[i] = new Array(BOARD_SIZE);
	}
	//create state table
	this.state = new Array(BOARD_SIZE);
	for (var i = 0; i < BOARD_SIZE; i++){
		this.state[i] = new Array(BOARD_SIZE);
	}

	this.toString = function(){
		var a = ""
		for (var i = 0; i < BOARD_SIZE; i++){
			for (var j = 0; j < BOARD_SIZE; j++){
				a += this.board[i][j];
			}
			a+= "<br>"
		}
		return a;
	}
	
	this.countAdjacentMines = function (x,y){
	//counts the number of adjacent mines to board[x][y]
		if (this.board[x][y] === "M"){
			return "M";
		}
		var adj = 0;
		if (this.checkForMine(x,y-1)){
			adj++;
		}
		if (this.checkForMine(x-1,y-1)){
			adj++;
		}
		if (this.checkForMine(x-1,y)){
			adj++;
		}
		if (this.checkForMine(x-1,y+1)){
			adj++;
		}
		if (this.checkForMine(x,y+1)){
			adj++;
		}
		if (this.checkForMine(x+1,y+1)){
			adj++;
		}
		if (this.checkForMine(x+1,y)){
			adj++;
		}
		if (this.checkForMine(x+1,y-1)){
			adj++;
		}
		return adj.toString();
	};

	this.checkForMine = function (x,y) {
	//checks for a mine at board[x][y]
	//if a mine exists there it returns true
	//returns false otherwise
		if (x < 0 || y < 0 ){
			return false;
		}
		if ( x > BOARD_SIZE-1 || y > BOARD_SIZE-1){
			return false;
		}
		return (this.board[x][y] === "M");
	};

	this.addNumbers = function(){
	//populates the board with mine adjacency numbers
		for (var i = 0; i < BOARD_SIZE; i++){
			for (var j = 0; j < BOARD_SIZE; j++){
				if (this.board[i][j] !== "M"){
					this.board[i][j] = this.countAdjacentMines(i,j);
				}
			}
		}
		
	};

	this.addMines = function(numMines){
	//populates the board with mines
		for (var i = 0; i < numMines; i++){
			var x = Math.floor(Math.random()*BOARD_SIZE);
			var y = Math.floor(Math.random()*BOARD_SIZE);
			if (this.board[x][y] !== "M") {
				this.board[x][y] = "M";
			} else {
				i--; //add another mine
			}
		}
	};

	this.clearBoard = function(){
	//sets all board squares to "C" to clear it
		for (var i = 0; i < BOARD_SIZE; i++){
			for (var j = 0; j < BOARD_SIZE; j++){
				this.board[i][j] = "C";
			}
		}
	};

	this.clearState = function(){
	//sets all state squares to "N" to clear it
		for (var i = 0; i < BOARD_SIZE; i++){
			for (var j = 0; j < BOARD_SIZE; j++){
				this.state[i][j] = "N";
			}
		}
	};

	this.newGame = function(){
	//clears the board/state and then populates it with mines and numbers
		this.clearBoard();
		this.clearState();
		this.addMines(MINE_COUNT);
		this.addNumbers();
	};

	this.flip = function(i,j){
	//flips the state of square at state[i][j] to "P" aka played iff it is notplayed yet aka "N"
	//if there are no adjacent mines, flips all adjacent squares
		if (i < 0 || j < 0 )
			return;
		if (i > BOARD_SIZE-1 || j > BOARD_SIZE-1)
			return;
		if (this.state[i][j] === "P")
			return;

		if (this.state[i][j] === "N")
			this.state[i][j] = "P";

		if (this.board[i][j] === "0"){
			this.flip(i,j-1);
			this.flip(i,j+1);
			this.flip(i-1,j);
			this.flip(i-1,j-1);
			this.flip(i-1,j+1);
			this.flip(i+1,j);
			this.flip(i+1,j-1);
			this.flip(i+1,j+1);
		}
	};

	this.getState = function(i,j){
		return this.state[i][j];
	};
	this.checkValidMove = function(i,j){
		return (this.state[i][j] === "N");
	};
	this.getBoardSize = function () {
		return BOARD_SIZE;
	};

	this.getSquare = function (i,j){
		return this.board[i][j];
	};
}


