///////////////////////////////////////////////
//
//  matching.js
//  Kevin Ackeral
//
//  This file contains code to run a two player
//  player card matching game. 
//
///////////////////////////////////////////////

const STATE_OFF = "off";
const STATE_LOGIN = "login";
const STATE_PLAY = "play";
const STATE_PAUSE = "pause";
const STATE_END = "end";

var MatchingGame =
{
    dimensions: {x:6, y:4},
    state: STATE_OFF,
    players:[],
    board: null,
    store: null,
    display: null,
    matches: 0,
    
    turn: 
    {
        currentPlayer:0,
        cardsPicked:0,
        cards:[null,null],
        
        initialize: function()
        {
            this.currentPlayer=0;
            this.cardsPicked=0;
            this.cards=[null,null];
        },
        
        nextPlayer: function()
        {
            this.currentPlayer = this.currentPlayer * -1 + 1;
            this.cardsPicked = 0;
            this.cards[0] = null;
            this.cards[1] = null;
        }
    },
    
    start: function()
    {
        display = new Renderer();
        this.store = new Storage();
        state = STATE_LOGIN;
        display.showLogin();
    },
    
    //this function adds players with a name given by a field on-page
    //to players. when two players are added, the game begins
    login: function()
    {   
        var username = document.getElementById("login_name").value;
        //validate the input
        if (username === "")
        {
            display.loginMessage("Please enter a valid name.")
        }
        else
        {
            switch (this.players.length)
            {
            case 0:
                display.loginMessage("Player two, please enter your name.")
                this.players.push(new Player(username));
                break;
            case 1:
                if (username === this.players[0].name)
                {
                    display.loginMessage(username + " is already playing! Enter a new name.")
                    break;
                }
                display.loginMessage("Player one, please enter your name.")
                this.players.push(new Player(username));
                display.hideLogin();
                this.play()
                break;
            default:
                break;
            }
            
            document.getElementById("login_name").value = "";
        }  
    },
    
    //this function initializes the game for actual play
    //
    play: function()
    {
        this.state = STATE_PLAY;
        
        //add and initialize board
        this.board = new Board(this.dimensions.x,this.dimensions.y);
        this.board.generate();
        
        //populate html table with tds to hold card images
        table = document.getElementById("gameboard_body");
        table.innerHTML = "";
        
        for(var i = 0; i < this.dimensions.y; i++)
        {
            var row = document.createElement("tr");
            
            for (var j = 0; j < this.dimensions.x; j++)
            {
                //create clickable container
                var data = document.createElement("td");
                data.setAttribute("onClick","MatchingGame.takeTurn(" + j + "," + i + ")");
                
                //create images, front and back, to add to td
                var backImg = document.createElement("img");
                backImg.setAttribute("src","images/card_back.png");
                
                var frontImg = document.createElement("img");
                frontImg.setAttribute("src","images/card_"+(this.board.cards[j][i].value + 1).toString()+".png");
                
                data.appendChild(frontImg);
                data.appendChild(backImg);
                row.appendChild(data);
            }
            
            table.appendChild(row);
        }
        //get correct images for table
        display.drawBoard(this.board);
        
        display.displayMessage(this.players[0].name + "'s turn");
        display.drawPlayerScore(this.players);
        
        //get and show highscores
        var table = this.store.getScores();
        display.drawHighscore(table);
        
        //make game visible
        display.showGame();
    },
    
    //this function handles the user clicking on a card
    takeTurn: function(x,y)
    {
        //if the game is paused to look at the results
        //move the game along
        if(this.state === STATE_PAUSE)
        {
            display.drawBoard(this.board);
            display.displayMessage(this.players[this.turn.currentPlayer].name + "'s turn");
            this.state = STATE_PLAY;
            
            return null;
        }
        
        
        //only do something if we are in the play stage
        if(this.state != STATE_PLAY)
        {
            return null;
        }
        
        card = this.board.cards[x][y];
        
        //don't do anything if card is already flipped
        if(card.visible === true)
        {
            return null;
        }
        
        card.visible = true;
        
        //update screen
        display.drawBoard(this.board);
        
        //add card to list of cards picked up this turn
        this.turn.cards[this.turn.cardsPicked] = card;
        this.turn.cardsPicked++;
       
        
        //compare if two are picked
        if (this.turn.cardsPicked >= 2)
        {
            //match!
            if (this.turn.cards[0].value === this.turn.cards[1].value)
            {
                this.players[this.turn.currentPlayer].points++;
                display.displayMessage("It's a match! Click a card to continue.");
                
                this.matches++;
                
                //check to see if the game is over
                if(this.matches === this.dimensions.x * this.dimensions.y / 2)
                {
                    display.drawPlayerScore(this.players);
                    this.endGame();
                    return null;
                }
            }
            else
            {
                this.turn.cards[0].visible = false;
                this.turn.cards[1].visible = false;
                
                display.displayMessage("No match. Click a card to continue.");
            }
            
            this.turn.nextPlayer();
            this.state = STATE_PAUSE;
        }
        
        display.drawPlayerScore(this.players);
    },
    
    //this function displays a message regarding who won the game
    endGame: function()
    {
        this.state = STATE_END;
        
        //the wins fieldof these records indicates what will be added to the playsscore
        //deafult is 1 because when there's a tie, everyone wins :)
        var player0record = new Record(this.players[0].name,1,this.players[0].quit,1);
        var player1record = new Record(this.players[1].name,1,this.players[1].quit,1);
        
        if (player0record.quits == 1) player0record.wins = 0;
        if (player1record.quits == 1) player1record.wins = 0;
        
        if(this.players[0].points > this.players[1].points)
        {
            display.displayMessage(this.players[0].name + " won!");
            player1record.wins = 0;
        }
        else if(this.players[0].points < this.players[1].points)
        {
            display.displayMessage(this.players[1].name + " won!");
            player0record.wins = 0;
        }
        else
        {
            display.displayMessage("It was a draw!");
        }
        
        generateScore(player0record);
        generateScore(player1record);
        
        //load, update, save, and display highscores
        var table = this.store.getScores();
        
        var p0found = false;
        var p1found = false;
        
        for(var i = 0; i < table.length; i++)
        {
            if(!p0found)
            {
                if(compareStrings(table[i].name, player0record.name) === 0)
                {
                    table[i].wins += player0record.wins;
                    table[i].quits += player0record.quits;
                    table[i].plays += player0record.plays;
                    generateScore(table[i]);
                    p0found = true;
                }
            }
            if(!p1found)
            {
                if(compareStrings(table[i].name, player1record.name) === 0)
                {
                    table[i].wins += player1record.wins;
                    table[i].quits += player1record.quits;
                    table[i].plays += player1record.plays;
                    generateScore(table[i]);
                    p1found = true;
                }
            }
            if(p0found && p1found) break;
        }
        
        if(!p0found) table.push(player0record);
        if(!p1found) table.push(player1record);
        
        sortHighScores(table);
        
        this.store.saveScores(table);
        display.drawHighscore(table);
        
    },
    
    //this function return the game to the login state
    quit: function()
    {
        if(this.state === STATE_LOGIN || this.state === STATE_OFF)
        {
            return null;
        }
        
        if(this.state === STATE_PLAY || this.state === STATE_PAUSE)
        {
            this.players[this.turn.currentPlayer].points = 0;
            this.players[this.turn.currentPlayer].quit = 1;
            this.turn.nextPlayer();
            this.players[this.turn.currentPlayer].points = 1;
            
            this.endGame();
        }
        
        this.state = STATE_LOGIN;
        this.players = [];
        this.board =  null;
        this.matches = 0;
        this.turn.initialize();
        
        display.hideGame();
        display.showLogin();
    }
};



//start the code!
window.onload = function()
{
    MatchingGame.start();
}