///////////////////////////////////////////////
//
//  renderer.js
//  Kevin Ackeral
//
//  This file contains a class that can be used
//  to render the matching game
//
///////////////////////////////////////////////

function Renderer()
{
    this.showLogin = function()
    {
        document.getElementById("login_section").style.display="block";
    };
    this.hideLogin = function()
    {
        document.getElementById("login_section").style.display="none";
    };
    this.loginMessage = function(text)
    {
        document.getElementById("login_title").innerHTML = text;
    };
    
    this.showGame = function()
    {
        document.getElementById("gameboard_section").style.display="block";
        document.getElementById("scoreboard_section").style.display="block";
        document.getElementById("options_section").style.display="block";
    };
    
    this.hideGame = function()
    {
        document.getElementById("gameboard_section").style.display="none";
        document.getElementById("scoreboard_section").style.display="none";
        document.getElementById("options_section").style.display="none";
    };
    
    this.drawBoard = function(board)
    {
        rows = document.getElementById("gameboard_body").childNodes;
        
        for(var i = 0; i < rows.length; i++)
        {
            cards = rows[i].childNodes;
            
            for(var j = 0; j < cards.length; j++)
            {
                var cardImages = cards[j].childNodes;
                
                if(board.cards[j][i].visible)
                {
                    cardImages[0].style.display = "block";
                    cardImages[1].style.display = "none";
                }
                else
                {
                    cardImages[1].style.display = "block";
                    cardImages[0].style.display = "none";
                }
            }
        }
    };
    
    this.displayMessage = function(message)
    {
        document.getElementById("gameboard_message").innerHTML = message;
    };
    
    this.drawPlayerScore = function(players)
    {
        table = document.getElementById("session_score");
        table.innerHTML = "";
        
        for(var i = 0; i < players.length; i++)
        {
            var tr = document.createElement("tr");
            var name = document.createElement("td");
            name.innerHTML = players[i].name;
            var points = document.createElement("td");
            points.innerHTML = players[i].points;
            tr.appendChild(name);
            tr.appendChild(points);
            table.appendChild(tr);
        }
    };
    
    this.drawHighscore = function(scores)
    {
        table = document.getElementById("highscore_body");
        table.innerHTML = "";
        
        var tooltipContainer = document.getElementById("tooltips");
        
        if(!tooltipContainer)
        {
            tooltipContainer = document.createElement("div");
            tooltipContainer.setAttribute("id","tooltips");
            
            document.body.appendChild(tooltipContainer);
        }
        
        tooltipContainer.innerHTML = "";
        
        for(var i = 0; i < scores.length; i++)
        {
            var tr = document.createElement("tr");
            var name = document.createElement("td");
            name.innerHTML = scores[i].name.toString();
            name.setAttribute("id",i.toString() + "_highscore");
            
            var tooltip = generateTooltip(scores[i]);
            tooltip.setAttribute("id",i.toString() + "_tooltip");
            tooltipContainer.appendChild(tooltip);
            var score = document.createElement("td");
            score.innerHTML = scores[i].score.toFixed(2);
            tr.appendChild(name);
            tr.appendChild(score);
            table.appendChild(tr);
            name.addEventListener("mouseover",showTooltip,true);
            name.addEventListener("mouseout",hideTooltip,true);
        }
    };
    
}

function generateTooltip(record)
{
    var tooltip = document.createElement("div");
    tooltip.className = "tooltip";
    tooltip.innerHTML = "<p>Plays: " + record.plays.toString() +
                        "<br>Wins: " + record.wins.toString() +
                        "<br>Quits: " + record.quits.toString() + "</p>";
    
    return tooltip;
}

function showTooltip(e)
{
    //console.log("in");
    var tooltip = document.getElementById(parseInt(e.target.getAttribute("id")).toString() + "_tooltip");
    if (tooltip)
    {
        tooltip.style.top = e.clientY.toString()+"px";
        tooltip.style.display = "block";
        tooltip.style.opacity = "1";
    }
}

function hideTooltip(e)
{
    //console.log("out");
    var tooltip = document.getElementById(parseInt(e.target.getAttribute("id")).toString() + "_tooltip");
    if (tooltip)
    {
        tooltip.style.opacity = "0";
    }
}

function Board(width,height)
{
    this.width = width;
    this.height = height;
    
    this.cards = [];
    
    
    //Populate cards with a width by height 2d array of
    //card objects
    this.generate = function()
    {
        //create array to keep track of how many instances
        //of each card value there are
        var numCardFaces = this.width * this.height / 2; 
        var cardInstances = new Array(numCardFaces);
        for (var i = 0; i < numCardFaces; i++)
        {
            cardInstances[i] = 0;
        }
        
        //create array
        for(var i = 0; i < this.width; i++)
        {
            var column = [];
            
            for(var j = 0; j < this.height; j++)
            {  
                //give a value for this card
                var cardValue = Math.floor(Math.random()*numCardFaces);
                
                //check if there are already two of this card
                while(cardInstances[cardValue] >= 2)
                {
                    cardValue = Math.floor(Math.random()*numCardFaces);
                }
                cardInstances[cardValue]++;
                
                var card = new Card(cardValue);
                column.push(card);
            }
            
            this.cards.push(column);
        }
    };
}