///////////////////////////////////////////////
//
//  images.js
//  Kevin Ackeral
//
//  This file contains a class that can be used
//  to save and retrieve highscore records from
//  local storage
//
///////////////////////////////////////////////

const HIGHSCORE = "matching_game_highscore_table";

function Storage()
{
    this.getScores = function()
    {
        var table = localStorage.getItem(HIGHSCORE);
        if (! table) {
            table = new Array();
        }
        else {
            table = JSON.parse(table);
        }
        return table;
    }
    
    this.saveScores = function(table)
    {
        localStorage.setItem(HIGHSCORE, JSON.stringify(table));
    }

}

//records that will be stored
function Record(name, wins, quits, plays)
{
    this.wins = wins;
    this.quits = quits;
    this.plays = plays;
    this.name = name;
    this.score = 0;
}

function generateScore(rec)
{
    if (rec.plays === 0)
    {
        rec.score = 0;
    }
    else
    {
        rec.score = (rec.wins / rec.plays) * 10;
    }
}

function compareStrings(a, b)
{
    return a.toLowerCase().localeCompare(b.toLowerCase());
}

function compareRecords(a, b)
{
    if (b.score - a.score === 0)
    {
        return compareStrings(a.name, b.name);
    }
    return b.score - a.score;
}

function sortHighScores(table)
{
    table.sort(compareRecords);
}