/*
 * Game rendering engine
 */
function Renderer () {
    this.displayCount = 0;
    this.hideLogin = function () {
        var loginElem = document.getElementById("login");
        loginElem.innerHTML = "";
    };	

    this.render = function (game) {
        if (game.players.length < game.playerCnt) {
            this.renderPlayer();
        } else {
            this.renderBoard(game.board);
            this.renderStats();
            if (game.disc === null)
                game.disc = game.nextTurn();   
            this.renderPiece(game.disc);
        }
    };

    this.clear = function(game) {
        for (var i=0; i<game.dim.nX; i++) {
            for (var j=0; j<game.dim.nY; j++) {
                var elemID = "cell"+ i + "_" + j;
                var cellElem = document.getElementById(elemID);
                cellElem.innerHTML = "";
            }
        }
    };

    this.drawCircle = function(ctx, x , y) {
        ctx.beginPath();
        ctx.arc(x, y, 40, 0 , 2 * Math.PI, false);
        ctx.globalAlpha = 0.75;
        ctx.fillStyle = 'white';
        ctx.fill();
        ctx.lineWidth = 5;
        ctx.strokeStyle = 'steelblue';
        ctx.stroke();
        ctx.closePath;
    };

    this.renderPiece = function(piece) {
        canvas = document.getElementById("discCanvas");
        context = canvas.getContext("2d");
        context.clearRect(0,0,900,110);
        context.beginPath();
        context.arc(piece.x, piece.y, piece.radius, 0 , 2 * Math.PI, false);
        context.fillStyle = piece.colour;
        context.fill();
        context.lineWidth = 5;
        context.strokeStyle = 'dark' + piece.colour;
        context.stroke();
    };

    this.renderBoard = function() {
        // show gameZone
        var gameZone = document.getElementById('gamezone');
        gameZone.style.display = 'block';
        // hide login
        var rightPanel = document.getElementById('rightpanel');
        rightPanel.style.display = 'none';
        
        canvas = document.getElementById("boardCanvas");
        context = canvas.getContext("2d");
        context.globalAlpha = 0.75;
        context.fillStyle = "steelblue";

        context.beginPath();
        context.fillRect(25, 10, 750, 100, 50);
        context.fillRect(25, 130, 750, 650, 50);
        context.stroke();
        context.closePath();
        
        var yCoord = 200;

        for (var i=0;i<6;i++) {
            var xCoord = 100;

            for (var j=0;j<7;j++) {
                this.drawCircle(context, xCoord, yCoord);
                xCoord += 100;
            }
            yCoord += 100;
        }
        
        for (var i=0; i<7; i++) {
            for (var j=0; j<6; j++) {
                if (game.board.slots[i][j] !== null)
                    this.renderPiece(game.board.slots[i][j]);
            }
        }
        context.font = "bold 30px sans-serif";
        context.textAlign = "center";
        context.fillText(game.board.message, 365, 75);
    };

    this.renderStats = function() {
        // show gameZone
        var gameStats = document.getElementById('stats');
        gameStats.style.display = 'block';
        var html = '<section>Game Statistics'
                 + '<table>';
        
        for (var i=0;i<game.players.length;i++) {
    		player = game.players[i];
            html += '<tr><th colspan=4>Player ' + player.name + '</th></tr>'
                  + '<tr><th>wins:</th><td>' + player.wins + '</td>'
                  + '<th>losses:</th><td>' + player.losses + '</td></tr>'
                  + '<tr><th>games:</th><td>' + player.games + '</td>'
                  + '<th>score:</th><td>' + player.score + '</td></tr>';
        }
        html += '</table></section>' 
             +  '<section>Leaderboard'
             + '<table>'
             + '<tr><th>Player</th><th>Score</th><th>Games</th><th>Wins</th><th>Losses</th></tr>'
        
        var leaders = storage.loadAll();
        leaders.sort(sortScore);

        for (var i=0;i<leaders.length;i++) {
    		player = leaders[i];
            html += '<tr><td>' + player.name 
                  + '</td><td>' + player.score 
                  + '</td><td>' + player.games 
                  + '</td><td>' + player.wins 
                  + '</td><td>' + player.losses + '</td></tr>';
        }
        html += '</table></section>';
        gameStats.innerHTML = html;
    };
    
    function sortScore(a, b) {
        if (a.score < b.score) return 1;
        if (a.score > b.score) return -1;
        return 0;
    }

    this.animatePiece = function(piece,tx,ty) {
        while ((piece.x !== tx) || (piece.y !== ty)) {
            if (piece.x < tx)
                piece.x += 1;
            else if (piece.x > tx)
                piece.x -= 1;

            if (piece.y < ty)
                piece.y += 1;
            else if (piece.y > ty)
                piece.y -= 1;

            canvas = document.getElementById("discCanvas");
            context = canvas.getContext("2d");
            context.clearRect(0,110,900,800);
            this.renderPiece(piece);
        }
    };

    this.renderPlayer = function() {
        var gameZone = document.getElementById("gamezone");		
        var gameStats = document.getElementById("stats");		
        gameZone.style.display = 'none';	
        gameStats.style.display = 'none';	

        var loginElem = document.getElementById("login");
        loginElem.innerHTML = '<section>'
                            + '<h2>Player' + (game.players.length + 1) + '</h2>'
                            + '<form>'
                            + '<label for="userName">User Name</label>'
                            + '<input id="userName" type="text"><br>'
                            + '<label for="colour">Game Piece Colour</label><br>'
                            + '<select id="colour"><option value="">Select a colour</option>'
                            + '<option value="red">Red</option>'
                            + '<option value="blue">Blue</option>'
                            + '<option value="green">Green</option>'
                            + '<option value="yellow">Yellow</option>'
                            + '<option value="pink">Pink</option>'
                            + '<option value="purple">Purple</option>'
                            + '<option value="orange">Orange</option>'
                            + '<option value="magenta">Magenta</option></select>'
                            + '<div class="clearfix">' 
                            + '<div class="button">'
                            + '<span class="icon" onclick="game.login()">Y</span></div>'
                            + '<div class="clearfix">'
                            + '</form></section>';
                    
    };

    this.displayTurn = function (game) {
        var turnElem = document.getElementById("turnManager");
        turnElem.innerHTML = "Turn: " + game.players[game.turnManager.currentPlayerID].name;
    };

    this.displayWinner = function () {
        game.board.message = game.currentPlayer.name + " is the winner in "
                           + game.plays + " moves";
        renderer.displayCount++;
        if (renderer.displayCount > 25) {
            clearInterval(renderer.displayWinner);
        } else {
            for (var j=0;j<4;j++) {
                game.board.slots[game.row[j][1]][game.row[j][2]].colour = renderer.rndColor();
            }
        }
        renderer.renderBoard();
    };

    this.displayTie = function () {
        game.board.message = "Game tied - please refresh to play again";
        renderer.renderBoard();
    };
    
    this.rndColor = function() {
        var hex = '0123456789ABCDEF'.split(''),
            color = '#', i;
        for (i = 0; i < 6 ; i++) {
            color = color + hex[Math.floor(Math.random() * 16)];
        }
        return color;
    }
}