/*
	The Game is Defined below.
	Player definition is inside game and looks like: {name:"",wins:0,games:0,forfeits:0}
	The board object is the canvas object imgcanvas.
*/
var Game = function(){
/**********************************************
******************VARIABLES********************
**********************************************/
	//Image Definitions
	var imgtree = new Image(); imgtree.src = 'tree.png';
	var imgtext = new Image(); imgtext.src = 'treeTextStart.png';
	var imgleaves = new Image(); imgleaves.src = 'treeLeaves.png';
	var imggrass = new Image(); imggrass.src = 'treeGrass.png';
	var imgfallingbranches = new Image();
	
	//Settings for drawing the branches
	var settings = {};
	settings.length = 7;
	settings.segmentHeight = 8;
	settings.segmentWidth = 16;
	settings.timer = 2000;
	settings.interval = 50;
	
	//Holds who's turn it is (0 or 1 position in array)
	var turn;
	//Remember if it is a forfeit or loss.
	var forfeit;
	//Rename "this" so it can be used in functions where this is already used.
	var GAME = this;
	//Holds Player Objects {name:"",wins:0,games:0,forfeits:0}
	//Max 2 players
	GAME.players = new Array();
	
	function newPlayerObj(){return {name:'',wins:0,games:0,forfeits:0,losses:0,points:0};}
	
	if(!localStorage.players){
		var p = newPlayerObj();
		p.name = "Player 1";
		GAME.players.push(p);
		var p = newPlayerObj();
		p.name = "Player 2";
		GAME.players.push(p);
		localStorage.players = JSON.stringify(GAME.players);
	}else{
		GAME.players = JSON.parse(localStorage.players);
	}
	//Hold timers for animations
	GAME.animate;
	//Hold timeouts
	GAME.timeouts = [];
	//Remember what row a player started on
	GAME.rowClicked;
	//Contains layers which are animated
	GAME.layers = new Array();
	//Offset for animations
	GAME.slide = 0;
	//Control button from page
	GAME.button = document.getElementsByClassName("btn")[0];
	//Table elements;
	GAME.table = document.getElementsByTagName("table")[0];
	//Message area from page
	GAME.msgbox = document.getElementById("msg");

/**********************************************
*****************Game Logic********************
**********************************************/
	
	function onBranchClick(e){
		//If row hasn't been clicked yet, set this row to be the row clicked
		if(GAME.rowClicked === '')
			GAME.rowClicked = e.parentNode.id;
		onBranchMouseOver(e);
	}
	
	function onBranchMouseOver(e){
		//Remove this branch if clicked and is in the row that was clicked
		if(GAME.rowClicked === e.parentNode.id){
			var context = e.getContext('2d');
			context.clearRect(0, 0, e.width, e.height);
			//animate leaves
			dropLeaves(e);
			//Remove bindings
			e.onmousedown = null;
			e.onmouseover = null;
			//Subtract the branch from the total
			e.parentNode.branches--;
		}
		//console.log(this.parentNode.branches);
		//this.parentNode.removeChild(this);	
	}
	
	function nextTurn(){
		//Check if player lost
		if(loss()){
			//Slide image back
			var img = new Image();
			img.src = copyBranches();
			animateTree(img,"GAME.init();","down");
			
			//remove all branches canvases
			for(i=0;i<3;i++)
				document.getElementById('brow'+i).innerHTML = '';
				
			//Disable Forfeit Button
			disableForfeit();					
			
			//Announce winner/loser
			animateTree(img,"print('Congratulations "+GAME.players[(turn+1)%2].name+"!');dispTree('win');","down");
			console.log("Player "+GAME.players[turn].name+" loses.");
			console.log("Player "+GAME.players[(turn+1)%2].name+" wins.");
			//Record who won
			recordStats((turn+1)%2);
		}else{
			//Reset values and change turns
			turn = (turn+1)%2;
			print("It is "+GAME.players[turn].name+"'s turn.");
			GAME.rowClicked = '';
		}
	}
	
	GAME.forfeit = function(){
		disableForfeit();
		forfeit = true;
		//Display forfeit
		print(GAME.players[turn].name + " forfeits.");
		//record win
		recordStats((turn+1)%2);
		announceWinner();				
	};
	
/**********************************************
*****************Game Tools********************
**********************************************/

	function print(msg){GAME.msgbox.innerHTML = msg;}
	
	function enableForfeit(){
		GAME.button.innerHTML = 'Forfeit';
		GAME.button.style.background = "rgb(255, 255, 255)";
		GAME.button.onclick = GAME.forfeit;
	}
	
	function disableForfeit(){
		GAME.button.innerHTML = 'New Game';
		GAME.button.style.background = "rgb(255, 200, 200)";
		GAME.button.onclick = GAME.init;
	}
	
	function loss(){
		//Return true if no branches
		var rows = document.getElementsByClassName('branchesRow');
		return(rows[0].branches === 0 && rows[1].branches === 0 && rows[2].branches === 0);
	}
	
	function recordStats(winner){
		//Change player's stats
		//Count 1 game for each
		GAME.players[0].games++;
		GAME.players[1].games++;
		
		//Count the win
		GAME.players[winner].wins++;
		
		//Count the loss for forfeit
		if(forfeit)
			GAME.players[(winner+1)%2].forfeits++;
		else
			GAME.players[(winner+1)%2].losses++;
	
		
		GAME.players[0].points = Math.floor(GAME.players[0].wins/GAME.players[0].games*Math.min(10,GAME.players[0].games));
		GAME.players[1].points = Math.floor(GAME.players[1].wins/GAME.players[1].games*Math.min(10,GAME.players[1].games));
	
		localStorage.players = JSON.stringify(GAME.players);
		displayStats();
	}
	
	//Gets the appropriate elements from the table and displays the saved stats
	function displayStats(){
		for(i=0;i<2;i++){
			GAME.table.getElementsByClassName("players")[i].innerHTML = '<a href="#openModal">'+GAME.players[i].name+'</a>';
			document.getElementsByClassName("playerInput")[i].value = GAME.players[i].name;
			GAME.table.getElementsByClassName("games")[i].innerHTML = GAME.players[i].games;
			GAME.table.getElementsByClassName("wins")[i].innerHTML = GAME.players[i].wins;
			GAME.table.getElementsByClassName("losses")[i].innerHTML = GAME.players[i].losses;
			GAME.table.getElementsByClassName("points")[i].innerHTML = GAME.players[i].points;
			GAME.table.getElementsByClassName("forfeits")[i].innerHTML = GAME.players[i].forfeits;
			/*if(GAME.players[i].games>0)
				GAME.table.getElementsByClassName("points")[i].innerHTML = Math.floor(GAME.players[i].wins/GAME.players[i].games*Math.min(10,GAME.players[i].games));
			else
				GAME.table.getElementsByClassName("points")[i].innerHTML = 0;*/
		}
	}
	
	function displayBranches(){
		reset();
		var rows = document.getElementsByClassName('branchesRow');
		for(i=0;i<rows.length;i++)
			rows[i].style.visibility = "";
			
		//Enable forfeit button
		enableForfeit();
		//No row has been selected
		GAME.rowClicked = '';
		//Randomize who's turn it is;
		turn = Math.floor(Math.random()*2);
		//Display turn to players
		print(GAME.players[turn].name+" goes first.");
		//Bind mouse click release to end turn.
		document.onmouseup = function(){
			//Check if player has played
			if(GAME.rowClicked !== '' && GAME.rowClicked !== null){
				GAME.rowClicked = null;
				nextTurn();
			}
		}
	}
		
	function announceWinner(){
		//Slide image back
		var img = new Image();
		img.src = copyBranches();
		//remove all branches canvases
		for(i=0;i<3;i++)
			document.getElementById('brow'+i).innerHTML = '';
		animateTree(img,"print('Congratulations "+GAME.players[(turn+1)%2].name+"!');dispTree('win');","down");
	}
	
	function announceLoser(){
		//Slide image back
		print('Sorry '+GAME.players[turn].name+' :(');
		dispTree('lose');
	}
	
	function changePlayers(p1,p2){
		//Can't play vs self
		if(p1===p2){
			print("Both players cannot be the same person.");
			return;
		}
		for(n=2;n>0;n--){
			//Check to see if player already exists, if so, move them to the front.
			for(i=0;i<GAME.players.length;i++){
				if(GAME.players[i].name === eval('p'+n)){
					GAME.players.unshift(GAME.players.splice(i,1)[0]);
					break;
				}
			}
			//If the player wasn't found, create them.
			if(GAME.players[0].name !== eval('p'+n)){
				var p = newPlayerObj();;
				p.name = eval('p'+n);
				GAME.players.unshift(p);
			}
		}
		localStorage.players = JSON.stringify(GAME.players);
		displayStats();
	}
	//Initialize size of canvas and game div
	GAME.init = function(){
		document.getElementsByClassName("loginBtn")[0].onclick = function(){
			changePlayers(document.getElementsByClassName("playerInput")[0].value,document.getElementsByClassName("playerInput")[1].value);
			location.hash = '#close';
		};
		var canvas = document.getElementById('imgCanvas');
		canvas.width = 310;
		canvas.height = 310;
		var gameArea = document.getElementById('game');
		gameArea.style.width = "310px";
		gameArea.style.height = "310px";
		disableForfeit();
		displayStats();
		print('');
		forfeit = false;
		reset();
		var rows = document.getElementsByClassName('branchesRow');
		for(i=0;i<rows.length;i++)
			rows[i].innerHTML = "";
		while(GAME.timeouts.length > 0){
			clearTimeout(GAME.timeouts.pop());
		}
		dispTree("game");
	};
	
	//clean board before reload the scores
	function cleanBoard(){
		var tab = document.getElementById("boardTab");
		var tr = tab.getElementsByTagName('tr');
		var i = 1;
		while(i<tr.length){
			tab.deleteRow(1);
		}
	}
    
	//onclick call to insert information into leader board
	document.getElementById("leadBoard").onclick = function(){
		cleanBoard();
		var i = 0;
		while(i<GAME.players.length){
			var tab = document.getElementById("boardTab");
			tr=document.createElement("tr");
			tab.appendChild(tr);
			td=document.createElement("td");
			tr.appendChild(td);
			td.innerHTML=GAME.players[i].name;
			td=document.createElement("td");
			tr.appendChild(td);
			td.innerHTML=GAME.players[i].games;
			td=document.createElement("td");
			tr.appendChild(td);
			td.innerHTML=GAME.players[i].wins;
			td=document.createElement("td");
			tr.appendChild(td);
			td.innerHTML=GAME.players[i].losses;
			td=document.createElement("td");
			tr.appendChild(td);
			td.innerHTML=GAME.players[i].forfeits;
			td=document.createElement("td");
			tr.appendChild(td);
			td.innerHTML=GAME.players[i].points;
			i++;
			}
	};
	
	
/**********************************************
********Game Drawings & Animations*************
**********************************************/
	//Draws the game and binds game mechanics.
	//Graphics are not displayed until displayBranches() is called.
	function drawBranches(){
		//Draw the branches
		//rows
		for(i=0;i<3;i++){
			//individual branch
			for(j=(7-i*2);j>0;j--){
				//Create the canvas for each branch
				var canvas = document.createElement('canvas');
				//Find the div it belongs in
				var branchesArea = document.getElementById('brow'+i);
				//Hide div
				branchesArea.style.visibility = "hidden";
				//Save the number of branches on the container
				branchesArea.branches = (7-i*2);
				//Give canvas a unique ID
				canvas.id = "stick"+i+j;
				//Set style information
				canvas.width = settings.segmentWidth*2;
				canvas.height = settings.length*settings.segmentHeight+15;
				//initialize the number of leaves
				canvas.leaves = 0;
				//Add canvas to page
				branchesArea.appendChild(canvas);
				//Draw the branch on the canvas
				drawBranch(canvas,settings.length,canvas.width/2,canvas.height,3);
				//Bind click functionality
				canvas.onmousedown = function(){onBranchClick(this);};
				//When hovering on a branch, if it's row has been clicked, remove it.
				canvas.onmouseover = function(){onBranchMouseOver(this);};
			}
		}
	}
	
	//Resets ALL canvases and animations
	function reset(){
		//Clear main canvas
		var canvas = document.getElementById('imgCanvas');
		var context = canvas.getContext('2d');
		context.clearRect(0, 0, canvas.width, canvas.height);
		
		//Stop tree and reset animation value
		GAME.slide = 0;
		stopTree();
		
		//Remove any animation layers
		while(GAME.layers.length){
			var layer = GAME.layers.pop();
			layer.c.parentNode.removeChild(layer.c);
		}
	}
	
	//Displays tree with layers
	//Takes function to be executed at the end of the tree shake animation
	function dispTree(screen){
		reset();
		var canvas;
		var context;
		var img = new Image();
		
		//Draw Tree
		canvas = document.getElementById('imgCanvas');
		context = canvas.getContext('2d');
		context.drawImage(imgtree, 5, 5);
		
		drawLayer(imggrass,5,5,1); //Draw Background Grass
		drawLayer(imgleaves,5,5,3); //Draw Top Leaves
		switch(screen){
			case "win":
				img.src = makeWin();
				if(forfeit)
					animateTree(img,"animateTree(new Image(),'GAME.init()','down')","up");
				else
					animateTree(img,"animateTree(new Image(),'announceLoser();','down')","up");						
				break;
			case "lose":
				img.src = makeLose();
				animateTree(img,"animateTree(new Image(),'GAME.init()','down')","up");
				break;
			//Unknown screen
			default:
				canvas = drawLayer(imgtext,5,5,4); //Draw Text
				//Bind click to start to animating the tree and initializing the game
				canvas.onclick = function(){
					drawBranches();
					img.src = copyBranches();
					animateTree(img,"displayBranches()","up");
				};
				break;
		}
	}
	
	//Draws Branches.
	//requires a canvas, length of segments, starting positions x and y, and how many (max 3) appendages)
	function drawBranch(canvas,l,xStart,yStart,branches){
		//if length is less than 1, we are past the end of the branch. return.
		if(l<1)	return;
		var context = canvas.getContext("2d");
		//initialize branch with valid javascript code
		var branch = "1;";
		var x;
		var y;
		
		//Draw a line
		context.beginPath();
		context.moveTo(xStart, yStart);
		for (var i=0; i<l; i++){
			//Initialize values for this segment
			var offset = (3-branches)*2;
			var randomSpace = 10-offset;
			y = yStart-(i+1)*settings.segmentHeight;
			//change starting position if branch (changes if a side branch)
			if(branches > 0){
				x = xStart;
			}else{
				if(branches === -1)
					x = xStart+settings.segmentWidth/3
				else
					x = xStart-settings.segmentWidth/3;
			}
			//Thinner line if approaching end of segments
			context.lineWidth = 4-(settings.length-l+i)/2;
			context.lineTo(x, y);
			context.stroke();
			//Check for appendage
			if(Math.random() > .75 && branches > 1){
				//Add valid javascript to branch variable which draws another branch
				//We do this since we cannot make the call now, as we are in the middle of drawing
				branch += "drawBranch(canvas,"+(l-i-1)+","+x+","+y+","+(-(--branches))+");";
			}
		}
		//Draw a leaf on the end of a branch maybe.
		if(Math.random() > .2){
			drawLeaf(canvas,x,y);
		}
		//Executes javascript code from a string.
		//Draws side branches
		eval(branch);
	}

	//Takes Canvas and x,y positions. Draws a leaf, returns rotation leaf recieved.
	function drawLeaf(canvas,x,y){
		//Remember how many leaves are stored
		canvas.leaves++;
		//Give it a random rotation. But still pointing up
		var rotation = Math.random()*3.5-2;
		context = canvas.getContext("2d");
		//Centre where leaf will be drawn
		context.save();
		context.translate(x,y);
		//Draw Curve
		context.lineWidth = 1;
		context.beginPath();
		context.rotate(rotation);
		context.moveTo(0, -1);
		context.bezierCurveTo(0, -15, 10, -5, 0, 0);
		context.stroke();
		context.closePath();
		//Colour white
		context.fillStyle = '#FFFFFF';
		context.fill();
		context.restore();
		return rotation;
	}
	
	//Redraws the leaf as it falls
	//Takes in canvas and array of leaf with positional data.
	function animateLeaves(canvas,leaves){
		//Check if every leaf is done animating
		var finished = true;
		for(i=0;i<leaves.length;i++)
			finished = finished && leaves[i].done;
		
		//If every leaf is done animating, end the timer.
		if(finished){
			clearInterval(canvas.animateTimer);
			return;
		}
		//Draw leaves
		context = canvas.getContext("2d");
		context.clearRect(0, 0, canvas.width, canvas.height);
		for(i=0;i<leaves.length;i++){
			//Check y position of leaf. 
			//If room to fall, randomly lower move and rotate the leaf
			//If too far down, end it's animation.
			if(leaves[i].y < canvas.height-15){
				leaves[i].x += Math.random()*2-1;
				leaves[i].y += Math.random()*5;
				leaves[i].r += Math.random()*.2;
			}else{
				leaves[i].done = true;
			}
			context.save();
			context.translate(leaves[i].x,leaves[i].y);
			context.lineWidth = 1;
			context.beginPath();
			context.rotate(leaves[i].r);
			context.moveTo(0, -1);
			context.bezierCurveTo(0, -15, 10, -5, 0, 0);
			context.stroke();
			context.closePath();
			context.fillStyle = '#FFFFFF';
			context.fill();
			context.restore();				
		}
	}
	
	//Initially creates leaves to be dropped.
	//Takes in canvas which holds the animation
	function dropLeaves(canvas){
		//Canvas has no leaves. Return.
		if(canvas.leaves === 0) return;
		//Create array to hold leaf objects
		var leaves = new Array();
		//Initialize leaves at top, random left to right position.
		//Draw leaf, save rotational data.
		for(i=canvas.leaves;i>0;i--){
			var x = (settings.segmentWidth*2-15)*Math.random()+5;
			var y = 15;
			var r = drawLeaf(canvas,x,y);
			//Add leaf data to array
			leaves.push({x:x,y:y,r:r,done:false});
		}
		//Animate on this canvas, the leaves we just created
		canvas.animateTimer = setInterval(function(){animateLeaves(canvas,leaves);},50);
	}			
	
	function animateTree(img,fun,dir){
		var canvas = document.getElementById('imgCanvas');
		var context = canvas.getContext('2d');
		//Copy current contents of the canvas
		
		if(dir === "up"){
			reset();
			//Draw Tree
			context.drawImage(imgtree, 5, 5);
			drawLayer(imggrass,5,5,1); //Draw Background Grass
			drawLayer(imgleaves,5,5,3); //Draw Top Leaves
		
			//Shake Tree
			//Save timer in animate variable
			var interval = settings.timer;
			//create picture of falling branches
			var tmpCanvas = document.createElement('canvas');
			var tmpContext = tmpCanvas.getContext('2d');
			//Generate some branches to fall.
			for(i=0;i<20;i++){
				var x = ((((Math.random()*170)-70)%300)+300)%300;
				var y = Math.random()*100+70;
				drawBranch(tmpCanvas,settings.length,x,y,3);
			}
			imgfallingbranches.src = tmpCanvas.toDataURL();
			
			GAME.animate = setInterval(function(){
				interval -= settings.interval*2;
				shakeTree(interval);
			}, settings.interval*2);
			
			//Stop Shaking the Tree after 2 seconds. Start next animation.
			GAME.timeouts.push(setTimeout(function(){
				stopTree();
				//Remove extra leaves
				layer = GAME.layers.pop();
				layer.c.parentNode.removeChild(layer.c);
				//Start sliding tree animation
				GAME.animate = setInterval(function(){
					slideTreeUp(img,fun);
				}, settings.interval);						
			}, settings.timer));
		}else if(dir === "down"){
			var shift = false;
			//Copy the currently displayed picture
			if(!img.src){
				shift = true;
				img.src = canvas.toDataURL();
			}
			reset();
			//Draw Tree
			context.drawImage(imgtree, 5, -300);
			drawLayer(imggrass,5,-300,1); //Draw Background Grass
			drawLayer(imgleaves,5,-300,3); //Draw Top Leaves
			//Start sliding tree animation
			GAME.animate = setInterval(function(){
				slideTreeDown(img, fun, shift);
			}, settings.interval);	
		}
	}
	
	//Creates a new layer on the canvas. Takes in an image to be displayed and a x,y, and z-index value.
	function drawLayer(img, x, y, z){
		var gameArea = document.getElementById('game');
		var canvas = document.createElement('canvas');
		canvas.width = 310;
		canvas.height = 310;
		context = canvas.getContext('2d');
		context.drawImage(img, x, y);
		gameArea.appendChild(canvas);
		canvas.style.top = "0px";
		canvas.style.position = "absolute";
		canvas.style.zIndex = ''+z;
		canvas.style.webkitBorderRadius = "8px";
		canvas.style.borderRadius = "8px";
		canvas.style.MozBorderRadius = "8px";
		canvas.style.OBorderRadius = "8px";
		canvas.style.msBorderRadius = "8px";
		GAME.layers.push({c:canvas,i:img});
		return	canvas;
	}
	
	//Shakes the entire game window
	//Drop Branches
	function shakeTree(interval){
		//SHAKE
		var game = document.getElementById('game');
		//get new random rotations and x and y values within a range (x,y range is 8, rotation range is 4)
		var rx = Math.floor(Math.random() * (8+1)) -8/2,
			ry = Math.floor(Math.random() * (8+1)) -8/2,
			rrot = Math.floor(Math.random() * (4+1)) -4/2;		
		rx = (rx === 0 && 8 !== 0) ? ((Math.random() < .5) ? 1 : -1) : rx;
		ry = (ry === 0 && 8 !== 0) ? ((Math.random() < .5) ? 1 : -1) : ry;

		//Change style of div to new positions
		game.style.position = 'relative';
		game.style.left = rx+'px';
		game.style.top = ry+'px';
		game.style.webkitTransform = 'rotate('+rrot+'deg)';
		game.style.MozTransform = 'rotate('+rrot+'deg)';
		game.style.msTransform = 'rotate('+rrot+'deg)';
		game.style.OTransform = 'rotate('+rrot+'deg)';
		game.style.transform = 'rotate('+rrot+'deg)';
		
		//DROP BRANCHES
		//clear canvas
		var canvas = document.getElementById('imgCanvas');
		var context = canvas.getContext('2d');
		context.clearRect(0, 0, canvas.width, canvas.height);
		//Draw images which are on the canvas
		context.drawImage(imgtree, 5, 5);//Draw Tree
		context.drawImage(imgfallingbranches,5,(-600*interval/settings.timer)+400);//Draw branches falling
		context.clearRect(0, 0, canvas.width, 5); //Remove line at top
	}
	
	//Reset position and rotation of game window
	function stopTree(){
		//Remove animation timers
		clearInterval(GAME.animate);
		//Reset style of game div
		var game = document.getElementById('game');
		game.style.left = '0px';
		game.style.top = '0px';
		game.style.webkitTransform = 'rotate(0deg)';
		game.style.MozTransform = 'rotate(0deg)';
		game.style.msTransform = 'rotate(0deg)';
		game.style.OTransform = 'rotate(0deg)';
		game.style.transform = 'rotate(0deg)';
	}
	
	//Takes in a function to be executed while sliding tree
	function slideTreeUp(img,fun){
		var canvas;
		var context;
		
		//Iterate sliding value
		GAME.slide += 10;
		//Animate the layers of the game
		for(i=0;i<GAME.layers.length;i++){
			var obj = GAME.layers[i];
			canvas = obj.c;
			context = canvas.getContext('2d');
			context.clearRect(0, 0, canvas.width, canvas.height);
			//If the image is the grass, animate faster than things in the front
			if(obj.i.src.substr(-13,13) == 'treeGrass.png')
				context.drawImage(obj.i, 5, -(GAME.slide*1.5));
			else
				context.drawImage(obj.i, 5, -GAME.slide);
		}
		
		//Animate base tree
		canvas = document.getElementById('imgCanvas');
		context = canvas.getContext('2d');
		context.clearRect(0, 0, canvas.width, canvas.height);
		context.drawImage(imgtree, 5, -GAME.slide);
		context.drawImage(img, 5, 305-GAME.slide);
		
		
		//If slide off screen, stop the timer.
		if(GAME.slide > 300){
			GAME.slide = 0;
			clearInterval(GAME.animate);
			if("displayBranches()"===fun){
				eval(fun);
			}else{
				GAME.timeouts.push(setTimeout(function(){
					//console.log(fun);
					eval(fun);
				}, settings.timer));
			}
		}
	}
	
	//Takes in a function to be executed while sliding tree
	function slideTreeDown(img,fun,shift){
		var canvas;
		var context;
		
		//Iterate sliding value
		GAME.slide += 10;
		//Animate the layers of the game
		for(i=0;i<GAME.layers.length;i++){
			var obj = GAME.layers[i];
			canvas = obj.c;
			context = canvas.getContext('2d');
			context.clearRect(0, 0, canvas.width, canvas.height);
			//If the image is the grass, animate faster than things in the front
			if(obj.i.src.substr(-13,13) == 'treeGrass.png')
				context.drawImage(obj.i, 5, (GAME.slide*1.5)-300*1.5);
			else
				context.drawImage(obj.i, 5, GAME.slide-300);
		}
		
		//Animate base tree
		canvas = document.getElementById('imgCanvas');
		context = canvas.getContext('2d');
		context.clearRect(0, 0, canvas.width, canvas.height);
		context.drawImage(imgtree, 5, GAME.slide-300);
		if(shift)
			context.drawImage(img, 0, GAME.slide-5);
		else
			context.drawImage(img, 5, GAME.slide);			
		
		//If slide off screen, stop the timer.
		if(GAME.slide > 300){
			GAME.slide = 0;
			clearInterval(GAME.animate);
			eval(fun);
		}
	}
	
	function copyBranches(){
		var canvas = document.createElement('canvas');
		var context = canvas.getContext('2d');
		canvas.width = 300;
		canvas.height = 300;
		for(i=0;i<3;i++){
			var row = document.getElementById('brow'+i).children;
			for(j=0;j<row.length;j++){
				context.drawImage(row[j],32*j+38+32*i,103*i+25);
			}
		}
		var img = new Image(); img.src = canvas.toDataURL();
		return canvas.toDataURL();
	}
				
	function makeLose(){
		reset();
		//var canvas = document.createElement('canvas');
		var canvas = document.getElementById('imgCanvas');
		var context = canvas.getContext('2d');
		context.save();
		//Y
		context.translate(70,80);
		context.rotate(-Math.PI/8);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/4);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/8);
		context.translate(0,60);
		drawBranch(canvas,settings.length,0,0,3);
		//O
		context.translate(40,0);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(55,0);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(-Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/2);
		//U
		context.translate(30,-60);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,-55);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,115);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(55,0);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		//L
		context.translate(-240,90);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		//O
		context.translate(70,0);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(55,0);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(-Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/2);
		//S
		context.translate(20,-55);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(0,-55);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(55,55);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(-Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/2);
		//E
		context.translate(30,-55);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,-55);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(0,110);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.restore();
		
		return canvas.toDataURL();
	};
	
	function makeWin(){
		reset();
		//var canvas = document.createElement('canvas');
		var canvas = document.getElementById('imgCanvas');
		var context = canvas.getContext('2d');
		context.save();
		//Y
		context.translate(70,80);
		context.rotate(-Math.PI/8);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/4);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/8);
		context.translate(0,60);
		drawBranch(canvas,settings.length,0,0,3);
		//O
		context.translate(40,0);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(55,0);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,55);
		context.rotate(-Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI/2);
		//U
		context.translate(30,-60);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,-55);
		context.rotate(Math.PI);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI);
		context.translate(0,115);
		context.rotate(Math.PI/2);
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(-Math.PI/2);
		context.translate(55,0);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		//W
		context.translate(-185,90);
		context.rotate(Math.PI*(15/16));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-15/16));
		context.translate(12,55);
		context.rotate(Math.PI*(15/16));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-15/16));
		context.translate(12,55);
		context.rotate(Math.PI*(1/16));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-1/16));
		context.translate(12,-55);
		context.rotate(Math.PI*(15/16));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-15/16));
		context.translate(12,55);
		context.rotate(Math.PI*(1/16));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-1/16));
		context.translate(12,-55);
		context.rotate(Math.PI*(1/16));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-1/16));
		//I
		context.translate(30,55);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		//N
		context.translate(30,55);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		context.rotate(Math.PI*(7/8));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-7/8));
		context.translate(22,55);
		context.rotate(Math.PI*(7/8));
		drawBranch(canvas,settings.length,0,0,3);
		context.rotate(Math.PI*(-7/8));
		context.translate(22,55);
		drawBranch(canvas,settings.length,0,0,3);
		context.translate(0,-55);
		drawBranch(canvas,settings.length,0,0,3);
		context.restore();
		
		return canvas.toDataURL();
	};
}