function Renderer() {
   var self = this;
   self.animationInProgress = false;
   self.canvasouter = null;
   self.canvas = null;
   self.columnWidth = 0;
   self.rowHeight = 0;
   self.numColumns = 0;
   self.numRows = 0;

   this.init = function (game) {
      self.canvasouter = document.getElementById(game.board.name);
      
      self.canvasouter.style.height = "100%";
      self.canvasouter.style.width = "75%";
      self.canvasouter.height = self.canvasouter.clientHeight;
      self.canvasouter.width = self.canvasouter.clientWidth;
      
      self.canvas = self.canvasouter.getContext('2d');
      self.numColumns = game.board.numColumns;
      self.numRows = game.board.numRows;
      self.columnWidth = self.canvasouter.width / game.board.numColumns;
      self.rowHeight = self.canvasouter.height / game.board.numRows;
      self.board = game.board;
      self.game = game;
      
      self.updateGameInfo();
      self.redrawGrid();

      self.canvasouter.addEventListener('mousemove', function (evt) {
         if (self.animationInProgress) {
            return;
         }
         self.canvas.clearRect(0, 2, self.canvas.canvas.width, self.canvas.canvas.height / self.board.numRows - 5);
         self.mousePos = self.relMouseCoords(evt);

         //self.writeMessage(self, "", self.board);

         self.drawCircle(self.mousePos.x, self.rowHeight / 2, self.game.currentPlayer);

      }, false);

      self.canvasouter.addEventListener('click', function (evt) {
         if (self.animationInProgress) 
         {
            return;
         }

         self.mousePos = self.relMouseCoords(evt);

         var columnnum = Math.floor((self.mousePos.x / self.columnWidth));

         self.game.Status = self.board.returnGameState(columnnum, self.game.currentPlayer, self.game.lastMove);

         self.game.lastMove = {row: self.game.Status.row, column: columnnum};
         if (self.game.Status.row === 0) {
            return;
         }

         self.animationInProgress = true;
         window.requestAnimFrame = (function (callback) {
         return window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame || function(callback) { window.setTimeout(callback, 1000 / 60); }; })();

         self.animate(30, self.mousePos.x, self.rowHeight / 2, self.mousePos.x, (self.rowHeight * self.game.Status.row + self.rowHeight / 2), 0, 0, .2);

      }, false);
      
   }
	
	this.animate = function (numFrames, circleCurrentX, circleCurrentY, circleEndX, circleEndY, circleVelocityX, circleVelocityY, easeValue) {
		// update

		// clear
		self.canvas.clearRect(0, 0, self.canvas.canvas.width, self.canvas.canvas.height);

		// draw stuff
		self.redrawGrid();
		self.redrawPieces();

		var dX = circleEndX - circleCurrentX;
		var dY = circleEndY - circleCurrentY;

		circleVelocityX = dX * easeValue;
		circleVelocityY = dY * easeValue;

		circleCurrentX += circleVelocityX;
		circleCurrentY += circleVelocityY;

		self.drawCircle(circleCurrentX, circleCurrentY, self.game.currentPlayer);


		// request new frame
		requestAnimFrame(function () 
		{
			numFrames--;
			if (numFrames > 0) 
			{
				self.animate(numFrames, circleCurrentX, circleCurrentY, circleEndX, circleEndY, circleVelocityX, circleVelocityY, easeValue);
			} 
			
			else 
			{
				if (self.game.Status.Gamestate === "Victory") 
				{
				   self.game.localStorageConnection.addEntry(self.game.players[0].playername , self.game.players[1].playername , self.game.players[self.game.currentPlayer - 1].playername , self.game.players[self.game.currentPlayer - 1].playername);
				   
					self.writeMessage(self.canvasouter, "Winner: " + self.game.players[self.game.currentPlayer - 1].playername + ((self.game.currentPlayer==1)?" (Red!)" : " (Gold!)"));

				} 
				
				else if (self.game.Status.Gamestate === "Draw!") 
				{
					self.writeMessage(self.canvasouter, "Draw!  You're both losers!");
					
				   self.game.localStorageConnection.addEntry(self.game.players[0].playername , self.game.players[1].playername , "Both losers!" , null);

				} 
				else 
				
				{
					self.animationInProgress = false;

					if (self.game.currentPlayer === 1) 
					{
						self.game.currentPlayer = 2;
					} 
					else 
					{
						self.game.currentPlayer = 1;
					}
               self.updateGameInfo();
					self.drawCircle(self.mousePos.x, self.rowHeight / 2, self.game.currentPlayer);
				}
			}
		});
	}

   this.redrawGrid = function () {
	
      self.canvas.lineWidth = 2;
      self.canvas.strokeStyle = '#003300';

      self.canvas.beginPath();
      self.canvas.moveTo(0, self.rowHeight);

      for (var i = 0; i < self.numColumns ; i++) {
         self.canvas.lineTo(self.columnWidth * i, self.canvasouter.height);
         self.canvas.moveTo((i + 1) * self.columnWidth, self.rowHeight);
      }

      self.canvas.stroke();
      self.canvas.closePath();


      self.canvas.beginPath();
      self.canvas.moveTo(0, 0);

      for (var i = 0; i < self.numRows ; i++) {
         self.canvas.lineTo(self.canvasouter.width, i * self.rowHeight);
         self.canvas.moveTo(0, (i+1) * self.rowHeight);
      }

      self.canvas.stroke();
      self.canvas.closePath();
   }

   this.writeMessage = function (canvas, msg, board) {
      var context = self.canvas; 
      context.clearRect(0, 2, self.canvas.width, self.canvas.height / self.board.numRows - 5);
      context.font = '18pt Calibri';
      context.fillStyle = 'black';
      context.fillText(msg, 10, 25);
   }

   this.drawCircle = function (x, y, currentPlayer) {
      var columnnum = Math.floor((x / self.game.renderer.columnWidth))
      self.canvas.beginPath();
      self.canvas.arc(columnnum * (self.game.renderer.columnWidth) + (self.game.renderer.columnWidth / 2), y, (Math.min(self.game.renderer.columnWidth , self.game.renderer.rowHeight)/2)*.65, 0, 2 * Math.PI, false);

      if (currentPlayer !== 'undefined') {
         if (currentPlayer == 1) {
            self.canvas.fillStyle = 'red';
         } else {
            self.canvas.fillStyle = 'yellow';
         }
      } else {
         if (currentPlayer == 1) {
            self.canvas.fillStyle = 'red';
         } else {
            self.canvas.fillStyle = 'yellow';
         }
      }

      self.canvas.fill();
      self.canvas.lineWidth = 5;
      self.canvas.strokeStyle = '#003300';
      self.canvas.stroke();
      self.canvas.closePath();
   }

   this.redrawPieces = function () {

      for (var i = 0 ; i < self.board.numColumns ; i++) {
         for (var j = 0 ; j < self.board.numRows ; j++) {

            if (i == self.game.lastMove.column && j == self.game.lastMove.row) {
               continue;
            }

            if (self.board.array[i][j] == 1) {
               self.drawCircle(self.columnWidth * i + self.columnWidth / 2, self.rowHeight * j + self.rowHeight / 2, 1);
            }

            else if (self.board.array[i][j] == 2) {
               self.drawCircle(self.columnWidth * i + self.columnWidth / 2, self.rowHeight * j + self.rowHeight / 2, 2);
            }
         }
      }
   }

   this.relMouseCoords = function (event) {
      if (event.offsetX !== undefined && event.offsetY !== undefined) { 
         // IE9 + Chrome
          return { x: event.offsetX, y: event.offsetY }; 
      } else {
          // Firefow
         return { x: event.layerX, y: event.layerY };
      }
   }

   this.drawWinner = function () {
      self.animationInProgress = true;  
   }

   this.updateGameInfo = function () {
      if (self.game.Status.Gamestate !== "Victory") { 
         var gameStateSpan = document.getElementById("gameState");
         gameStateSpan.innerHTML = "Game State: " + self.game.Status.Gamestate + "<br />";
         self.game.updatePlayerNames();
         var currentPlayerSpan = document.getElementById("currentPlayer");
         currentPlayerSpan.innerHTML = "Current Player: " + self.game.players[self.game.currentPlayer - 1].playername + "<br />";
      } else { 
         var gameStateSpan = document.getElementById("gameState");
         gameStateSpan.innerHTML = self.game.players[self.game.currentPlayer - 1].playername + " has " + self.game.Status.Gamestate + "<br />";
         var currentPlayerSpan = document.getElementById("currentPlayer");
         currentPlayerSpan.innerHTML = "";
      }
   }
   this.showRules = function () {
      var rules = document.getElementById("rules");
      
      if (rules.innerHTML === "") {
         rules.innerHTML = "Rules go here!@";
      } else {
         rules.innerHTML = "";
      }

   }
}
